Package: ocs4R
Version: 0.3.1
Date: 2026-04-29
Title: Interface to Open Collaboration Services (OCS) REST API
Authors@R: c(
  person("Emmanuel", "Blondel", role = c("aut", "cre"), email = "emmanuel.blondel1@gmail.com", comment = c(ORCID = "0000-0002-5870-5762")))
Maintainer: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
Depends: R (>= 3.3.0), methods
Imports: R6, openssl, curl,httr, jsonlite, XML, keyring
Suggests: testthat
Description: Provides an Interface to Open Collaboration Services 'OCS' (<https://www.open-collaboration-services.org/>) REST API.
License: MIT + file LICENSE
URL: https://github.com/eblondel/ocs4R
BugReports: https://github.com/eblondel/ocs4R/issues
LazyLoad: yes
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2026-04-29 21:33:36 UTC; emman
Author: Emmanuel Blondel [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5870-5762>)
Repository: CRAN
Date/Publication: 2026-04-29 22:10:02 UTC
Built: R 4.5.2; ; 2026-05-01 18:04:23 UTC; unix

Package: ocs4R
Version: 0.3.1
Date: 2026-04-29
Title: Interface to Open Collaboration Services (OCS) REST API
Authors@R: c(
  person("Emmanuel", "Blondel", role = c("aut", "cre"), email = "emmanuel.blondel1@gmail.com", comment = c(ORCID = "0000-0002-5870-5762")))
Maintainer: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
Depends: R (>= 3.3.0), methods
Imports: R6, openssl, curl,httr, jsonlite, XML, keyring
Suggests: testthat
Description: Provides an Interface to Open Collaboration Services 'OCS' (<https://www.open-collaboration-services.org/>) REST API.
License: MIT + file LICENSE
URL: https://github.com/eblondel/ocs4R
BugReports: https://github.com/eblondel/ocs4R/issues
LazyLoad: yes
RoxygenNote: 7.3.1
NeedsCompilation: no
Packaged: 2026-04-29 21:33:36 UTC; emman
Author: Emmanuel Blondel [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-5870-5762>)
Repository: CRAN
Date/Publication: 2026-04-29 22:10:02 UTC
Built: R 4.6.0; ; 2026-05-01 21:20:41 UTC; unix

# Generated by roxygen2: do not edit by hand

export(ocs4RLogger)
export(ocsApiSharingManager)
export(ocsApiUserProvisioningManager)
export(ocsApiWebdavManager)
export(ocsManager)
export(ocsRequest)
import(XML)
import(curl)
import(jsonlite)
import(keyring)
import(methods)
importFrom(R6,R6Class)
importFrom(httr,DELETE)
importFrom(httr,GET)
importFrom(httr,POST)
importFrom(httr,PUT)
importFrom(httr,add_headers)
importFrom(httr,authenticate)
importFrom(httr,content)
importFrom(httr,cookies)
importFrom(httr,handle)
importFrom(httr,headers)
importFrom(httr,http_status)
importFrom(httr,status_code)
importFrom(httr,upload_file)
importFrom(httr,with_verbose)
importFrom(openssl,base64_encode)

[![Build Status](https://github.com/eblondel/ocs4R/actions/workflows/r-cmd-check.yml/badge.svg?branch=master)](https://github.com/eblondel/ocs4R/actions/workflows/r-cmd-check.yml)
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/ocs4R)](https://cran.r-project.org/package=ocs4R)
[![cran checks](https://badges.cranchecks.info/worst/ocs4R.svg)](https://cran.r-project.org/web/checks/check_results_ocs4R.html)
[![Github_Status_Badge](https://img.shields.io/badge/Github-0.3.1-blue.svg)](https://github.com/eblondel/ocs4R)
[![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.3663614.svg)](https://doi.org/10.5281/zenodo.3663614)

**ocs4R – R Interface to Open Collaboration Services (OCS) REST APIs**

``ocs4R`` provides a R interface to the [Open Collaboration Services (OCS)](https://www.open-collaboration-services.org) REST APIs. ``ocs4R`` includes support for the below APIs:

* OCS Webdav API
* OCS Share API
* OCS User Provisioning API

``ocs4R`` constitutes the generic R client to access to OCS-compliant cloud platforms such as [ownCloud](https://owncloud.com/) and [Nextcloud](https://nextcloud.com/), either directly using [ocs4R](https://github.com/eblondel/ocs4R) or through product-specific ``ocs4R`` wrappers such as [ownCloud4R](https://github.com/eblondel/ownCloud4R) and [nextCloud4R](https://github.com/eblondel/nextCloud4R)

<a name="package_status"/>

Please check the [online documentation](https://github.com/eblondel/ocs4R/wiki) for more details!

## Sponsors

For ocs4R sponsoring/funding new developments, enhancements, support requests, please contact me by [e-mail](mailto:eblondel.pro@gmail.com).

## Citation

You can get the preferred citation by using the ``ocs4R`` Document Object Identifier (DOI): [![DOI](https://zenodo.org/badge/DOI/10.5281/zenodo.3663614.svg)](https://doi.org/10.5281/zenodo.3663614)

We thank in advance people that use ``ocs4R`` for citing it in their work / publication(s). 

#' ocsManager
#' @docType class
#' @export
#' @keywords ocs manager
#' @return Object of \code{\link[R6]{R6Class}} for modelling an ocsManager
#' @format \code{\link[R6]{R6Class}} object.

#' @examples
#' \dontrun{
#'    #Not Run:
#'    #Connect to an OCS API
#'    OCS <- ocsManager$new(url = ocs_url, user = ocs_user, pwd = ocs_pwd, logger = "DEBUG")
#'    version <- OCS$getVersion()
#'    caps <- OCS$getCapabilities()
#'    
#'    #OCS User Provisioning API
#'    #-------------------------------------
#'    #users
#'    users <- OCS$getUsers() #get users
#'    user <- OCS$getUser("admin") #get a user
#'    user.df <- OCS$getUser("admin", TRUE) #the same user as data.frame
#'    added <- OCS$addUser("john.doe", password = "ocs4john") #add a user
#'    disabled <- OCS$disableUser("john.doe") #disable a user
#'    enabled <- OCS$enableUser("john.doe") #enable auser
#'    edited <- OCS$editUser("john.doe", key = "display", value = "John Doe") #edit user
#'    #edit some user field
#'    edited2 <- OCS$editUserDisplayName("john.doe", displayName = "John Doe Jr.") 
#'    deleted <- OCS$deleteUser("john.doe")
#'    
#'    #groups
#'    admingroups <- OCS$getUserGroups("admin")
#'    groups <- OCS$getGroups()
#'    added <- OCS$addGroup("scientists") #add a new group
#'    sc_group <- OCS$getGroup("scientists") #get group details
#'    added <- OCS$addToGroup("john.doe", "scientists") #add user to group
#'    removed <- OCS$removeFromGroup("john.doe", "scientists") #remove user from group
#'    deleted <- OCS$deleteGroup("scientists")
#'    
#'    #OCS Webdav API
#'    #-------------------------------------
#'    #list files
#'    files <- OCS$listFiles()
#'    subfiles <- OCS$listFiles("Documents")
#'    #make collection
#'    OCS$makeCollection("myfolder")
#'    subfiles <- OCS$listFiles("myfolder")
#'    #upload a file?
#'    filename <- "magic.txt"
#'    file.create(filename); writeLines("ocs4R is great", filename)
#'    #we upload the file in 'Documents' folder
#'    OCS$uploadFile(filename, "/Documents")
#'    #check if file is uploaded
#'    OCS$listFiles('Documents')
#'    
#'    #OCS Sharing API
#'    #-------------------------------------
#'    #let's add a user with User provisioning API
#'    added <- OCS$addUser("john.doe", password = "ocs4john") #add a user
#'    #let's share the previously uploaded file with John Doe
#'    OCS$shareWithUser("/Documents", filename, "john.doe")
#' }
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#' 
ocsManager <-  R6Class("ocsManager",
  inherit = ocs4RLogger,
  lock_objects = FALSE,
  private = list(
    url = NULL,
    user = NULL,
    pwd = NULL,
    token =  NULL,
    cookies = NULL,
    version = NULL,
    capabilities = NULL,
    
    keyring_backend = NULL,
    keyring_service = NULL,
    
    #getPassword
    getPassword = function(){
      if(!is.null(private$keyring_backend)){
        private$keyring_backend$get(service = private$keyring_service, username = paste0(private$user,"_pwd"))
      }else{
        private$pwd
      }
    },
    
    #getToken (if existing)
    getToken = function(){
      token <- NULL
      if(!is.null(private$keyring_service)){
        keyring_token <- suppressWarnings(try(private$keyring_backend$get(service = private$keyring_service, username = paste0(private$user, "_token")), silent = TRUE))
        if(!is(keyring_token, "try-error")) token <- keyring_token
      }else{
        token <-private$token
      }
      return(token)
    },
    
    #checkAPIAvailability
    checkAPIAvailability = function(name, element){
      if(!private$capabilities[[element]]$api_enabled){
        errMsg <- sprintf("This method requires the 'OCS' REST API %s to be enabled.", name)
        errMsg <- paste(errMsg, "Please get in touch with your administrator")
        self$ERROR(errMsg)
        stop(errMsg)
      }
    },
    
    #check
    checkSharingAPIAvailability = function(){
      private$checkAPIAvailability("OCS Sharing API", "files_sharing")
    }
  ),
  public = list(
    #'@field apis list of APIs
    apis = list(),
    
    #'@description This method is used to instantiate an ocsManager. The user/pwd are
    #'mandatory in order to connect to 'ocs'. The logger can be either NULL, "INFO" 
    #'(with minimum logs), or "DEBUG" (for complete curl http calls logs).
    #'
    #'The \code{keyring_backend} can be set to use a different backend for storing 
    #'the user password with \pkg{keyring} (Default value is NULL, meaning the password 
    #'is stored as private field).
    #'
    #'@param url url
    #'@param user user
    #'@param pwd pwd
    #'@param logger logger type 
    #'@param keyring_backend keyring back-end. Default is \code{NULL}
    initialize = function(url, user, pwd, logger = NULL,
                          keyring_backend = NULL){
      super$initialize(logger = logger)
      private$url = url
      private$user <- user
      if(!is.null(keyring_backend)){
        if(!keyring_backend %in% names(keyring:::known_backends)){
          errMsg <- sprintf("Backend '%s' is not a known keyring backend!", keyring_backend)
          self$ERROR(errMsg)
          stop(errMsg)
        }
        
        private$keyring_backend <- keyring:::known_backends[[keyring_backend]]$new()
        private$keyring_service <- paste0("ocs4R@", url)
        private$keyring_backend$set_with_value(private$keyring_service, username = paste0(user,"_pwd"), password = pwd)
      }else{
        private$pwd <- pwd
      }
      
      #try to connect
      if(!startsWith(self$getClassName(), "ocsApi")){
        
        #test connection
        self$connect()
        
        #inherit managers methods (experimenting)
        list_of_classes <- rev(ls("package:ocs4R"))
        supportedManagers <- list_of_classes[regexpr("ocsApi.+Manager", list_of_classes)>0]
        for(manager in supportedManagers){
          class <- eval(parse(text=manager))
          man <- class$new(url, user, pwd, logger, keyring_backend)
          api_name <- tolower(unlist(strsplit(unlist(strsplit(manager, "ocsApi"))[2],"Manager"))[1])
          self$apis[[api_name]] <- man
          list_of_methods <- rev(names(man))
          for(method in list_of_methods){
            methodObj <- man[[method]]
            if(!(method %in% names(self)) && is(methodObj,"function")){1
              self[[method]] <- methodObj
              environment(self[[method]]) <- environment(self$connect)
            } 
          }
        }
      }
      invisible(self)
    },
    
    #'@description Method to connect to 'ocs' and set version/capabilities
    connect = function(){
      caps_req <- ocsRequest$new(
        type = "HTTP_GET", private$url, "ocs/v1.php/cloud/capabilities",
        private$user, pwd = private$getPassword(), 
        logger = self$loggerType
      )
      caps_req$execute()
      caps_resp <- caps_req$getResponse()
      
      req_cookies <- caps_resp$cookies
      cookies <- as.list(req_cookies$value)
      names(cookies) <- req_cookies$name
      if(length(cookies[names(cookies)=="XSRF-TOKEN"])>0){
        token <- cookies[names(cookies)=="XSRF-TOKEN"][[1]]
        if(!is.null(private$keyring_backend)){
          private$keyring_backend$set_with_value(private$keyring_service, username = paste0(private$user,"_token"), password = token) 
        }else{
          private$token = token
        }
      }
      cookies <- unlist(cookies[names(cookies)!="XSRF-TOKEN"])
      private$cookies <- paste0(sapply(names(cookies), function(cookiename){paste0(cookiename,"=",cookies[[cookiename]])}),collapse=";")
      
      keyring_token <- private$getToken()
      if(!is.null(keyring_token)){
        caps_req <- ocsRequest$new(
          type = "HTTP_GET", private$url, "ocs/v1.php/cloud/capabilities",
          private$user, token = keyring_token, cookies = private$cookies, 
          logger = self$loggerType
        )
        caps_req$execute()
        caps_resp <- caps_req$getResponse()
      }
      
      if(caps_resp$ocs$meta$status == "failure"){
        errMsg <- sprintf("Could not connect to ocs '%s': %s", private$url, caps_resp$ocs$meta$message)
        self$ERROR(errMsg)
        stop(errMsg)
      }
      if(caps_resp$ocs$meta$status == "ok"){
        self$INFO(sprintf("Successful connection to ocs '%s'!", private$url))
        private$version = caps_resp$ocs$data$version
        private$capabilities = caps_resp$ocs$data$capabilities
      }
    },
    
    #'@description  Get the 'ocs' server version
    getVersion = function(){
      return(private$version)
    },
    
    #'@description  Get the 'ocs' server capabilities
    getCapabilities = function(){
      return(private$capabilities)
    },
    
    #'@description Get the Webdav API manager
    #'@return an instance of \link{ocsApiWebdavManager}
    getAPIWebdavManager = function(){
      return(self$apis[["webdav"]])
    },
    
    #'@description Get the Sharing API manager
    #'@return an instance of \link{ocsApiSharingManager}
    getAPISharingManager = function(){
      return(self$apis[["sharing"]])
    },
    
    #'@description Get the User Provisioning API manager
    #'@return an instance of \link{ocsApiUserProvisioningManager}
    getAPIUserProvisioningManager = function(){
      return(self$apis[["userprovisioning"]])
    }
    
  )
)

# test_manager.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for OCS REST API manager
#=======================
require(ocs4R, quietly = TRUE)
require(testthat)

context("manager")

test_that("OCS API manager - version",{
  version <- OCS$getVersion()
  expect_equal(names(version)[1:5], c("major", "minor", "micro", "string", "edition"))
})

test_that("OCS API manager - capabilities",{
  caps <- OCS$getCapabilities()
  expect_true(all(c("core", "dav", "files", "files_sharing", "notifications") %in% names(caps)))
})

#' ocsRequest
#'
#' @docType class
#' @export
#' @keywords ocs request
#' @return Object of \code{\link[R6]{R6Class}} for modelling a generic 'ocs' web-service request
#' @format \code{\link[R6]{R6Class}} object.
#'
#' @section Methods:
#' \describe{
#'  \item{\code{new(type, url, request, user, pwd, token, cookies,
#'                  format, namedParams, content, contentType, filename, logger, ...)}}{
#'    This method is used to instantiate a object for doing an 'ocs' web-service request
#'  }
#'  \item{\code{getRequest()}}{
#'    Get the request payload
#'  }
#'  \item{\code{getRequestHeaders()}}{
#'    Get the request headers
#'  }
#'  \item{\code{getStatus()}}{
#'    Get the request status code
#'  }
#'  \item{\code{getResponse()}}{
#'    Get the request response
#'  }
#'  \item{\code{getException()}}{
#'    Get the exception (in case of request failure)
#'  }
#'  \item{\code{getResult()}}{
#'    Get the result \code{TRUE} if the request is successful, \code{FALSE} otherwise
#'  }
#' }
#' 
#' @note Abstract class used internally by \pkg{ocs4R}
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#'
ocsRequest <- R6Class("ocsRequest",
  inherit = ocs4RLogger,
  portable = TRUE,
  #private methods
  private = list(
    url = NA,
    type = NA,
    request = NA,
	  requestHeaders = NA,
    format = "json",
    namedParams = list(),
    content = NULL,
    contentType = "text/plain",
    filename = NULL,
	  status = NA,
    response = NA,
    exception = NA,
    result = NA,

    auth_scheme = "Basic",
    auth = NA,
    token = NULL,
    cookies = NULL,
    
    keyring_service = NULL,
    
    getUserAgent = function(){
      return(paste("ocs4R", packageVersion("ocs4R"), sep="-"))
    },

    #HTTP_GET
    #---------------------------------------------------------------
    HTTP_GET = function(url, request = NULL, format = "json", namedParams){
      req <- url
      if(!is.null(request)) req <- paste(url, request, sep = "/")
      namedParams$format = format
      namedParams <- namedParams[!sapply(namedParams, is.null)]
      paramNames <- names(namedParams)
      namedParams <- lapply(namedParams, function(namedParam){
        if(is.logical(namedParam)) namedParam <- tolower(as(namedParam, "character"))
        return(namedParam)
      })
      if(!endsWith(req,"?") && length(namedParams)>0) req <- paste0(req, "?")
      params <- paste(paramNames, namedParams, sep = "=", collapse = "&")
      req <- paste0(req, params)
      self$INFO(sprintf("HTTP/GET - Fetching %s", req))
      
      r <- NULL
      if(self$verbose.debug){
        r <- with_verbose(GET(req, handle = handle(''), add_headers(
          "User-Agent" = private$getUserAgent(),
          "Authorization" = private$auth,
          "X-XSRF-TOKEN" = private$token,
          "Set-Cookie" = private$cookies,
          "OCS-APIRequest" = "true"
        )))
      }else{
        r <- GET(req, handle = handle(''), add_headers(
          "User-Agent" = private$getUserAgent(),
          "Authorization" = private$auth,
          "X-XSRF-TOKEN" = private$token,
          "Set-Cookie" = private$cookies,
          "OCS-APIRequest" = "true"
        ))
      }
      
      responseContent <- NULL
      if(status_code(r)==200){
        self$INFO(sprintf("HTTP/GET - Successful request '%s'", req))
        contentType = "raw"
        if(!is.null(namedParams$format)) if(namedParams$format == "json") contentType = "application/json"
        responseContent <- httr::content(r, type = contentType, encoding = "UTF-8")
        if(contentType == "application/json") if(responseContent$ocs$meta$status == "failure"){
          errMsg <- sprintf("%s [status code = %s]", responseContent$ocs$meta$message, responseContent$ocs$meta$statuscode)
          self$ERROR(errMsg)
          stop(errMsg)
        }
      }
      if(status_code(r)==401){
        errMsg <- sprintf("HTTP/GET - Unauthorized request '%s' (insufficient privileges)", req)
        self$ERROR(errMsg)
        stop(errMsg)
      }
      if(status_code(r)==404){
        errMsg <- sprintf("HTTP/GET - File '%s' not found", req)
        self$ERROR(errMsg)
        stop(errMsg)
      }
      
      
      response <- list(request = request, requestHeaders = headers(r), cookies = cookies(r),
                       status = status_code(r), response = responseContent)
      return(response)
    },
    
    #HTTP_POST
    #---------------------------------------------------------------
    HTTP_POST = function(url, request = NULL, format = "json", namedParams = list(), content = "", contentType = "text/plain"){
      req <- url
      if(!is.null(request)) req <- paste(url, request, sep = "/")
      namedParams$format = format
      namedParams <- namedParams[!sapply(namedParams, is.null)]
      paramNames <- names(namedParams)
      namedParams <- lapply(namedParams, function(namedParam){
        if(is.logical(namedParam)) namedParam <- tolower(as(namedParam, "character"))
        return(namedParam)
      })
      if(!endsWith(req,"?") && length(namedParams)>0) req <- paste0(req, "?")
      params <- paste(paramNames, namedParams, sep = "=", collapse = "&")
      req <- paste0(req, params)
      
      self$INFO(sprintf("HTTP/POST - Sending request '%s'", req))
      
      #content
      body <- content
      
      r <- NULL
      if(self$verbose.debug){
        r <- with_verbose(POST(req, handle = handle(''), 
          add_headers(
            "User-Agent" = private$getUserAgent(),
            "Content-Type" = contentType,
            "Authorization" = private$auth,
            "X-XSRF-TOKEN" = private$token,
            "Set-Cookie" = private$cookies,
            "OCS-APIRequest" = "true"
          ),
          body = body
        ))
      }else{
        r <- POST(req, handle = handle(''), add_headers(
          "User-Agent" = private$getUserAgent(),
          "Content-Type" = contentType,
          "Authorization" = private$auth,
          "X-XSRF-TOKEN" = private$token,
          "Set-Cookie" = private$cookies,
          "OCS-APIRequest" = "true"), body = body)
      }
      
      responseContent <- NULL
      if(status_code(r)==200){
        self$INFO(sprintf("HTTP/POST - Successful request '%s'", req))
        responseContent <- httr::content(r, type = "application/json", encoding = "UTF-8")
        if(responseContent$ocs$meta$status == "failure"){
          errMsg <- sprintf("%s [status code = %s]", responseContent$ocs$meta$message, responseContent$ocs$meta$statuscode)
          self$ERROR(errMsg)
          stop(errMsg)
        }
      }
      if(status_code(r)==401){
        errMsg <- sprintf("HTTP/POST - Unauthorized request '%s' (insufficient privileges)", req)
        self$ERROR(errMsg)
        stop(errMsg)
      }
      
      response <- list(request = req, requestHeaders = headers(r), cookies = cookies(r),
                       status = status_code(r), response = responseContent)
      return(response)
    },
    
    #HTTP_PUT
    #---------------------------------------------------------------
    HTTP_PUT = function(url, request = NULL, format = "json", namedParams = list(), content = NULL, contentType = "application/x-www-form-urlencoded", filename = NULL){
      req <- url
      if(!is.null(request)) req = paste(url, request, sep="/")
      namedParams$format = format
      namedParams <- namedParams[!sapply(namedParams, is.null)]
      paramNames <- names(namedParams)
      namedParams <- lapply(namedParams, function(namedParam){
        if(is.logical(namedParam)) namedParam <- tolower(as(namedParam, "character"))
        return(namedParam)
      })
      if(!endsWith(req,"?") && length(namedParams)>0) req <- paste0(req, "?")
      params <- paste(paramNames, namedParams, sep = "=", collapse = "&")
      req <- paste0(req, params)

      self$INFO(sprintf("HTTP/PUT - Putting content at '%s'", req))
      
      #content
      query <- NULL
      body <- NULL
      if(missing(content) | is.null(content)){
        if(missing(filename) | is.null(filename)){
          stop("The filename must be provided")
        }
        content <- filename
        body <- httr::upload_file(filename)
      }else{
        query <- content
      }
      
      r <- NULL
      if(self$verbose.debug){
        r <- with_verbose(PUT(req, handle = handle(''), add_headers(
          "User-Agent" = private$getUserAgent(),
          "Authorization" = private$auth,
          "X-XSRF-TOKEN" = private$token,
          "Set-Cookie" = private$cookies,
          "OCS-APIRequest" = "true",
          "Content-Type" = contentType), body = body, query = query
        ))
      }else{
        r <- PUT(req, handle = handle(''), add_headers(
          "User-Agent" = private$getUserAgent(),
          "Authorization" = private$auth,
          "X-XSRF-TOKEN" = private$token,
          "Set-Cookie" = private$cookies,
          "OCS-APIRequest" = "true",
          "Content-Type" = contentType), body = body, query = query)
      }

      if(status_code(r) %in% c(200,201,204)){
        self$INFO(sprintf("HTTP/PUT - Content successfuly uploaded at '%s'", req))
      }else{
        if(status_code(r)==401){
          errMsg <- sprintf("HTTP/PUT - Unauthorized request '%s' (insufficient privileges)", req)
          self$ERROR(errMsg)
          stop(errMsg)
        }else{
          errMsg <- sprintf("HTTP/PUT - Error %s - ", status_code(r))
          self$ERROR(errMsg)
          stop(errMsg)
        }
      }
      
      response <- list(request = req, requestHeaders = headers(r), cookies = cookies(r),
                       status = status_code(r), response = content)
      return(response)
    },
    
    #HTTP_DELETE
    #---------------------------------------------------------------
    HTTP_DELETE = function(url, request = NULL, content = NULL, contentType = "text/plain"){
      req <- url
      if(!is.null(request)) req = paste(url, request, sep="/")
      
      self$INFO(sprintf("HTTP/DELETE - Deleting content at '%s'", req))
 
      r <- NULL
      if(self$verbose.debug){
        r <- with_verbose(DELETE(req, handle = handle(''), add_headers(
          "User-Agent" = private$getUserAgent(),
          "Content-Type" = contentType,
          "Authorization" = private$auth,
          "X-XSRF-TOKEN" = private$token,
          "Set-Cookie" = private$cookies,
          "OCS-APIRequest" = "true"
          ), query = content
        ))
      }else{
        r <- DELETE(req, handle = handle(''), add_headers(
          "User-Agent" = private$getUserAgent(),
          "Content-Type" = contentType,
          "Authorization" = private$auth,
          "X-XSRF-TOKEN" = private$token,
          "Set-Cookie" = private$cookies,
          "OCS-APIRequest" = "true"), query = content
        )
      }
      
      if(status_code(r)==201){
        self$INFO(sprintf("HTTP/DELETE - Successful deletion at '%s'", req))
      }
      if(status_code(r)==401){
        errMsg <- sprintf("HTTP/DELETE - Unauthorized request '%s' (insufficient privileges)", req)
        self$ERROR(errMsg)
        stop(errMsg)
      }
      
      response <- list(request = req, requestHeaders = headers(r), cookies = cookies(r),
                       status = status_code(r), response = content)
      return(response)
    },
    
    #WEBDAV_PROPFIND
    WEBDAV_PROPFIND = function(url, request, anonymous = FALSE){
      req <- paste(url, request, sep = "/")
      
      self$INFO(sprintf("WEBDAV/PROPFIND - Listing files at '%s'", req))
      
      h <- new_handle()
      handle_setopt(h, customrequest = "PROPFIND")
      if(!anonymous){
        headers <- list("OCS-APIRequest" = "true", "Authorization" = private$auth)
        if(!is.null(private$token)) headers <- c(headers, "X-XSRF-TOKEN" = private$token)
        if(!is.null(private$cookies)) headers <- c(headers, "Set-Cookie" = private$cookies)
        handle_setheaders(h, .list = headers)
      }
      response <- curl_fetch_memory(req, h)
      xml <- rawToChar(response$content)
      response <- xmlParse(xml, asText = TRUE)
      webdavNS <- c(d = "DAV:")
      base <- paste(paste("/", strsplit(req, "/")[[1]][-1:-3], sep="", collapse=""), "/", sep="")
      nodes <- getNodeSet(response, "//d:response")
      if(length(nodes)>0){
        self$INFO("WEBDAV/PROPFIND - Successful file listing!")
      }else{
        errMsg <- "WEBDAV/PROPFIND - Error while listing files"
        self$ERROR(errMsg)
        stop(errMsg)
      }
      
      result <- do.call("rbind", lapply(nodes, function(node){
        out_node <- data.frame(
          name = sub(base, "", URLdecode(xpathSApply(xmlDoc(node), "//d:href", namespaces = webdavNS, xmlValue))),
          resourceType = ifelse(length(xmlChildren(getNodeSet(xmlDoc(node), "//d:propstat/d:prop/d:resourcetype", namespaces = webdavNS)[[1]]))==0,"file","collection"),
          contentLength = {
            ct <- xpathSApply(xmlDoc(node), "//d:propstat/d:prop/d:getcontentlength", namespaces = webdavNS, xmlValue)
            if(length(ct)==0) ct <- NA
            ct
          },
          contentType = {
            ct <- xpathSApply(xmlDoc(node), "//d:propstat/d:prop/d:getcontenttype", namespaces = webdavNS, xmlValue)
            if(length(ct)==0) ct <- NA
            ct
          },
          size = {
            s = xpathSApply(xmlDoc(node), "//d:propstat/d:prop/d:getcontentlength", namespaces = webdavNS, xmlValue)
            s = as.numeric(s)
            if(length(s)==0) s <- NA
            s
          },
          quota = {
            q = xpathSApply(xmlDoc(node), "//d:propstat/d:prop/d:quota-used-bytes", namespaces = webdavNS, xmlValue)
            q = as.numeric(q)
            q <- if(length(q)==0) NA else q/1e6
            q
          },
          lastModified = {
            date = xpathSApply(xmlDoc(node), "//d:propstat/d:prop/d:getlastmodified", namespaces = webdavNS, xmlValue)
            date = gsub(" GMT", "", date)
            lctime <- Sys.getlocale("LC_TIME"); Sys.setlocale("LC_TIME", "C")
            date <- strptime(date, "%a, %d %b %Y %H:%M:%S")
            Sys.setlocale("LC_TIME", lctime)
            if(length(date)==0) date <- NA
            date
          },
          stringsAsFactors = FALSE
        )
        return(out_node)
      }))
      result <- result[result$name != "", ]
      result$lastModified <- as.POSIXlt(result$lastModified, origin = "1970-01-01")
      
      response <- list(request = req, requestHeaders = NA,
                       status = NA, response = result)
      return(response)
    },
    
    #WEBDAV_MKCOL
    WEBDAV_MKCOL = function(url, request){
      response <- NULL
      req <- paste(url, request, sep = "/")
      self$INFO(sprintf("WEBDAV/MKCOL - Creating collection '%s' at '%s'", request, req))
      h <- new_handle()
      handle_setopt(h, customrequest = "MKCOL")
      headers <- list("OCS-APIRequest" = "true", "Authorization" = private$auth)
      if(!is.null(private$token)) headers <- c(headers, "X-XSRF-TOKEN" = private$token)
      if(!is.null(private$cookies)) headers <- c(headers, "Set-Cookie" = private$cookies)
      handle_setheaders(h, .list = headers)
      response <- curl_fetch_memory(req, h)
      if(response$status_code==201){
        self$INFO(sprintf("WEBDAV/MKCOL - Successfuly created collection '%s'", request))
        response <- list(request = req, requestHeaders = NA,
                         status = response$status_code, response = response$url)
      }else{
        errMsg <- sprintf("WEBDAV/MKCOL - Error while creating collection '%s' at '%s' (Error %s)", request, req, response$status_code)
        self$ERROR(errMsg)
        stop(errMsg)
      }
      return(response)
    }
  ),
  
  #public methods
  public = list(
    
    #'@description This method is used to instantiate a object for doing an 'ocs' web-service request
    #'@param type type of request
    #'@param url url
    #'@param request request
    #'@param user user
    #'@param pwd pwd
    #'@param token token
    #'@param cookies cookies
    #'@param format format. Default is "json"
    #'@param namedParams a list of named parameters
    #'@param content content
    #'@param contentType content type. Default is "text/plain"
    #'@param filename file name
    #'@param logger logger
    #'@param ... additional parameters
    initialize = function(type, url, request,
                          user = NULL, pwd = NULL,
                          token = NULL, cookies = NULL,
                          format = "json",
                          namedParams = list(),
                          content = NULL, contentType = "text/plain", 
                          filename = NULL,
                          logger = NULL, ...) {
      super$initialize(logger = logger)
      private$type = type
      private$url = url
      private$keyring_service <- paste0("ocs4R@", url)
      private$request = request
      private$format = format
      private$namedParams = namedParams
      private$content = content
      if(type == "HTTP_PUT") contentType = "application/x-www-form-urlencoded"
      private$contentType = contentType
      private$filename = filename
      
      #authentication schemes
      if(!is.null(user)){
        #Basic authentication (user/pwd) scheme
        private$auth_scheme <- "Basic"
        private$auth <- paste(
          private$auth_scheme, 
          openssl::base64_encode(paste(user, pwd,sep=":"))
        )
      }
      private$token <- token
      private$cookies <- cookies
    },
    
    #'@description Executes the request
    execute = function(){
      
      req <- switch(private$type,
        "HTTP_GET" = private$HTTP_GET(
          url = private$url, 
          request = private$request,
          format = private$format,
          namedParams = private$namedParams
        ),
        "HTTP_POST" = private$HTTP_POST(
          url = private$url, 
          request = private$request,
          format = private$format,
          namedParams = private$namedParams,
          content = private$content,
          contentType = private$contentType
        ),
        "HTTP_PUT" = private$HTTP_PUT(
          url = private$url, 
          request = private$request,
          format = private$format,
          namedParams = private$namedParams,
          content = private$content,
          contentType = private$contentType,
          filename = private$filename
        ),
        "HTTP_DELETE" = private$HTTP_DELETE(
          url = private$url,
          request = private$request,
          content = private$content,
          contentType = private$contentType
        ),
        "WEBDAV_PROPFIND" = private$WEBDAV_PROPFIND(
          url = private$url,
          request = private$request, 
          anonymous = is.null(private$user)&is.null(private$token)&is.null(private$cookies)
        ),
        "WEBDAV_MKCOL" = private$WEBDAV_MKCOL(
          url = private$url, 
          request = private$request
        )
      )
      
      private$request <- req$request
      private$requestHeaders <- req$requestHeaders
      private$status <- req$status
      private$response <- req$response
    },
    
    #'@description Get request
    getRequest = function(){
      return(private$request)
    },
    
    #'@description Get request headers
    getRequestHeaders = function(){
      return(private$requestHeaders)
    },
    
    #'@description Get status
    getStatus = function(){
      return(private$status)
    },
    
    #'@description Get response
    getResponse = function(){
      return(private$response)
    },
    
    #'@description Get exception
    getException = function(){
      return(private$exception)
    },
    
    #'@description Get result
    getResult = function(){
      return(private$result)
    },
    
    #'@description Set result
    #'@param result result
    setResult = function(result){
      private$result = result
    }
    
  )
)

#' ocs4RLogger
#'
#' @docType class
#' @export
#' @keywords logger
#' @return Object of \code{\link[R6]{R6Class}} for modelling a simple logger
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @note Logger class used internally by ocs4R
#'
ocs4RLogger <-  R6Class("ocs4RLogger",
  portable = TRUE,
  public = list(
    #'@field verbose.info is info logger active?
    verbose.info = FALSE,
    #'@field verbose.debug is debug logger active?
    verbose.debug = FALSE,
    #'@field loggerType logger type
    loggerType = NULL,
    
    #'@description Initializes the logger
    #'@param logger the type of logger. Default is \code{NULL}, accepts \code{INFO} or \code{DEBUG}
    initialize = function(logger = NULL){
      
      #logger
      if(!missing(logger)){
        if(!is.null(logger)){
          self$loggerType <- toupper(logger)
          if(!(self$loggerType %in% c("INFO","DEBUG"))){
            stop(sprintf("Unknown logger type '%s", logger))
          }
          if(self$loggerType == "INFO"){
            self$verbose.info = TRUE
          }else if(self$loggerType == "DEBUG"){
            self$verbose.info = TRUE
            self$verbose.debug = TRUE
          }
        }
      }
    },
    
    #'@description Logger function
    #'@param type type of log
    #'@param text text
    logger = function(type, text){
      if(self$verbose.info){
        cat(sprintf("[ocs4R][%s] %s - %s \n", type, self$getClassName(), text))
      }
    },
    
    #'@description Logger to report information. Used internally
    #'@param text text
    INFO = function(text){self$logger("INFO", text)},
    
    #'@description Logger to report warnings. Used internally
    #'@param text text
    WARN = function(text){self$logger("WARN", text)},
    
    #'@description Logger to report errors Used internally
    #'@param text text
    ERROR = function(text){self$logger("ERROR", text)},
    
    #'@description Get class name
    #'@return the class name
    getClassName = function(){
      return(class(self)[1])
    },
    
    #'@description Get class
    #'@return the class
    getClass = function(){
      class <- eval(parse(text=self$getClassName()))
      return(class)
    }
    
  )
)

#' @name ocs4R
#' @aliases ocs4R-package
#' @aliases ocs4R
#' 
#' @importFrom R6 R6Class
#' @importFrom openssl base64_encode
#' @import methods
#' @import curl
#' @importFrom httr DELETE
#' @importFrom httr GET
#' @importFrom httr POST
#' @importFrom httr PUT
#' @importFrom httr add_headers
#' @importFrom httr authenticate
#' @importFrom httr content
#' @importFrom httr headers
#' @importFrom httr cookies
#' @importFrom httr http_status
#' @importFrom httr status_code
#' @importFrom httr with_verbose
#' @importFrom httr upload_file
#' @importFrom httr handle
#' @import jsonlite
#' @import XML
#' @import keyring
#' 
#' @title Interface to 'OCS' REST API
#' @description Provides an Interface to 'OCS' (<https://www.open-collaboration-services.org/>) REST API.
#'  
#'@author Emmanuel Blondel \email{emmanuel.blondel1@@gmail.com}
#'
"_PACKAGE"

#' ocsApiUserProvisioningManager
#' @docType class
#' @export
#' @keywords ocs manager userprovisioning api
#' @return Object of \code{\link[R6]{R6Class}} for modelling an ocsManager for Webdav API
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#' 
ocsApiUserProvisioningManager <-  R6Class("ocsApiUserProvisioningManager",
  inherit = ocsManager,
  public = list(
    
    #'@description Initialize manager
    #'@param url url
    #'@param user user
    #'@param pwd pwd
    #'@param logger logger
    #'@param keyring_backend backend to use with \pkg{keyring}. Default is \code{NULL}
    initialize = function(url, user, pwd, logger = NULL,
                          keyring_backend = 'env'){
      super$initialize(url, user, pwd, logger, keyring_backend)
    },
    
    #OCS USER PROVISIONING API
    #-------------------------------------------------------------------------------------------
    
    #Instruction set for users
    #-------------------------------------------------------------------------------------------
    
    #'@description Adds a user given a \code{userid} (required). All other fields (email, password, groups) are
    #'    optional for the user creation. Returns \code{TRUE} if the user is added, \code{FALSE} otherwise.
    #'@param userid user ID
    #'@param email email
    #'@param password user password
    #'@param groups groups
    addUser = function(userid, email = NULL, password = NULL, groups = NULL){
      request <- "ocs/v1.php/cloud/users"
      post_req <- ocs4R::ocsRequest$new(
        type = "HTTP_POST", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        content = list(
          userid = userid,
          email = email,
          password = password,
          groups = groups
        ),
        contentType = NULL,
        logger = self$loggerType
      )
      post_req$execute()
      post_req_resp <- post_req$getResponse()
      added <- post_req_resp$ocs$meta$statuscode == 100
      return(added)
    },
    
    #'@description Get the list of users. This method returns a vector of class 'character' giving 
    #'the user IDs available in the OCS cloud plateform.
    getUsers = function(){
      get_users <- ocs4R::ocsRequest$new(
        type = "HTTP_GET", private$url, "ocs/v1.php/cloud/users",
        private$user, pwd = private$getPassword(),
        token = private$getToken(), cookies = private$cookies,
        logger = self$loggerType
      )
      get_users$execute()
      get_users_resp <- get_users$getResponse()
      users <- unlist(get_users_resp$ocs$data$users)
      return(users)
    },
    
    #'@description Get the user details from its \code{userid}. If the argument \code{pretty} is set to TRUE,
    #'    this will return an object of class \code{data.frame}, otherwise (by default) it returns 
    #'    an object of class \code{list}.
    #'@param userid user ID
    #'@param pretty pretty
    getUser = function(userid, pretty = FALSE){
      get_user <- ocs4R::ocsRequest$new(
        type = "HTTP_GET", private$url, sprintf("ocs/v1.php/cloud/users/%s", userid),
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        logger = self$loggerType
      )
      get_user$execute()
      get_user_resp <- get_user$getResponse()
      user <- get_user_resp$ocs$data
      class(user$enabled) <- "logical"
      class(user$two_factor_auth_enabled) <- "logical"
      if(pretty){
        user <- as.data.frame(t(unlist(user)))
        row.names(user) <- 1
      }
      return(user)
    },
    
    #'@description Edits a user, identifier by a userid. The user property to be edited should be set using its
    #'    key (eg display) and the value to be modified for this key. Returns \code{TRUE} if the user 
    #'    is edited, \code{FALSE} otherwise.
    #'@param userid user ID
    #'@param key key
    #'@param value value
    editUser = function(userid, key, value){
      allowedKeys <- c("email", "quota", "display", "password")
      if(!key %in% allowedKeys){
        errMsg <- sprintf("Key should be among the following [%s]", paste(allowedKeys, collapse=","))
        self$ERROR(errMsg)
        stop(errMs)
      }
      request <- sprintf("ocs/v1.php/cloud/users/%s", userid)
      put_req <- ocsRequest$new(
        type = "HTTP_PUT", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        content = list(key = key, value = value),
        logger = self$loggerType
      )
      put_req$execute()
      put_req_resp <- put_req$getResponse()
      edited <- FALSE
      if(is(put_req_resp, "list")) if(!is.null(put_req_resp$key) && !is.null(put_req_resp$value)){
        if(put_req_resp$key == key && put_req_resp$value == value) edited <- TRUE
      }
      return(edited)
    },
    
    #'@description Edits a user display name.
    #'@param userid user ID
    #'@param displayName display name
    editUserDisplayName = function(userid, displayName){
      return(self$editUser(userid, key = "display", value = displayName))
    },
    
    #'@description Edits a user email
    #'@param userid user ID
    #'@param email email
    editUserEmail = function(userid, email){
      return(self$editUser(userid, key = "email", value = email))
    },
    
    #'@description Edits a user password
    #'@param userid user ID
    #'@param password password
    editUserPassword = function(userid, password){
      return(self$editUser(userid, key = "password", value = password))
    },
    
    #'@description Edits a user quota
    #'@param userid user ID
    #'@param quota quota
    editUserQuota = function(userid, quota){
      return(self$editUser(userid, key = "quota", value = quota))
    },
    
    #'@description Enables a user
    #'@param userid user ID
    #'@return \code{TRUE} if enabled, \code{FALSE} otherwise
    enableUser = function(userid){
      request <- sprintf("ocs/v1.php/cloud/users/%s/enable", userid)
      put_req <- ocsRequest$new(
        type = "HTTP_PUT", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        content = "",
        logger = self$loggerType
      )
      put_req$execute()
      return(TRUE)
    },
    
    #'@description Disables a user
    #'@param userid user ID
    #'@return \code{TRUE} if disabled, \code{FALSE} otherwise
    disableUser = function(userid){
      request <- sprintf("ocs/v1.php/cloud/users/%s/disable", userid)
      put_req <- ocsRequest$new(
        type = "HTTP_PUT", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        content = "",
        logger = self$loggerType
      )
      put_req$execute()
      return(TRUE)
    },
    
    #'@description Deletes a user
    #'@param userid user ID
    #'@return \code{TRUE} if deleted, \code{FALSE} otherwise
    deleteUser = function(userid){
      request <- sprintf("ocs/v1.php/cloud/users/%s", userid)
      delete_req <- ocsRequest$new(
        type = "HTTP_DELETE", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        logger = self$loggerType
      )
      delete_req$execute()
      return(TRUE)
    },
    
    #'@description Get user groups
    #'@param userid user ID
    #'@return the user groups
    getUserGroups = function(userid){
      get_usergroups <- ocs4R::ocsRequest$new(
        type = "HTTP_GET", private$url, sprintf("ocs/v1.php/cloud/users/%s/groups", userid),
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        logger = self$loggerType
      )
      get_usergroups$execute()
      get_usergroups_resp <- get_usergroups$getResponse()
      usergroups <- unlist(get_usergroups_resp$ocs$data$groups)
      return(usergroups)
    },
    
    #'@description Adds a user to a group.
    #'@param userid user ID
    #'@param groupid group ID
    #'@return \code{TRUE} if added, \code{FALSE} otherwise
    addToGroup = function(userid, groupid){
      request <- sprintf("ocs/v1.php/cloud/users/%s/groups", userid)
      post_req <- ocs4R::ocsRequest$new(
        type = "HTTP_POST", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        content = list(groupid = groupid),
        contentType = NULL,
        logger = self$loggerType
      )
      post_req$execute()
      post_req_resp <- post_req$getResponse()
      added <- post_req_resp$ocs$meta$statuscode == 100
      return(added)
    },
    
    #'@description Removes a user from a group.
    #'@param userid user ID
    #'@param groupid group ID
    #'@return \code{TRUE} if removed, \code{FALSE} otherwise
    removeFromGroup = function(userid, groupid){
      request <- sprintf("ocs/v1.php/cloud/users/%s/groups", userid)
      delete_req <- ocsRequest$new(
        type = "HTTP_DELETE", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        content = list(groupid = groupid),
        logger = self$loggerType
      )
      delete_req$execute()
      return(TRUE)
    },
    
    #'@description Creates a subadmin
    createSubadmin = function(){
      stop("'createSubadmin' method not yet implemented")
    },
    
    #'@description Removes a subadmin
    removeSubadmin = function(){
      stop("'removeSubadmin' method not yet implemented")
    },
    
    #'@description Get subadmin groups
    getSubadminGroups = function(){
      stop("'getSubadminGroups' method not yet implemented")
    },
    
    #Instruction set for groups 
    #-------------------------------------------------------------------------------------------
    
    #'@description Get the list of groups. This method returns a vector of class 
    #''character' giving the usergroups IDs
    #'@param search search
    #'@param limit limit
    #'@param offset offset
    getGroups = function(search = NULL, limit = NULL, offset = NULL){
      get_groups <- ocs4R::ocsRequest$new(
        type = "HTTP_GET", private$url, "ocs/v1.php/cloud/groups",
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        namedParams = list(search = search, limit = limit, offset = offset),
        logger = self$loggerType
      )
      get_groups$execute()
      get_groups_resp <- get_groups$getResponse()
      groups <- unlist(get_groups_resp$ocs$data$groups)
      return(groups)
    },
    
    #'@description Adds a group
    #'@param groupid group ID
    #'@return \code{TRUE} if added, \code{FALSE}
    addGroup = function(groupid){
      request <- "ocs/v1.php/cloud/groups"
      post_req <- ocs4R::ocsRequest$new(
        type = "HTTP_POST", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        content = list(groupid = groupid),
        contentType = NULL,
        logger = self$loggerType
      )
      post_req$execute()
      post_req_resp <- post_req$getResponse()
      added <- post_req_resp$ocs$meta$statuscode == 100
      return(added)
    },
    
    #'@description Gets a group
    #'@param groupid group ID
    #'@return the group as \code{list} including the group ID and the list of users
    getGroup = function(groupid){
      get_group <- ocs4R::ocsRequest$new(
        type = "HTTP_GET", private$url, sprintf("ocs/v1.php/cloud/groups/%s", groupid),
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        logger = self$loggerType
      )
      get_group$execute()
      get_group_resp <- get_group$getResponse()
      group <- list(id = groupid, users = unlist(get_group_resp$ocs$data$users))
      return(group)
    },
    
    #'@description Deletes a group
    #'@param groupid group ID
    #'@return \code{TRUE} if deleted, \code{FALSE}
    deleteGroup = function(groupid){
      request <- sprintf("ocs/v1.php/cloud/groups/%s", groupid)
      delete_req <- ocsRequest$new(
        type = "HTTP_DELETE", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        logger = self$loggerType
      )
      delete_req$execute()
      return(TRUE)
    },
    
    #'@description Get subadmins
    getSubadmins = function(){
      stop("'getSubadmins' method not yet implemented")
    }
    
  )
)

# test_api_user_provisioning.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for OCS manager User Provisioning API methods
#=======================
require(ocs4R, quietly = TRUE)
require(testthat)

context("api-user-provisioning")

test_that("User Provisioning API - getUsers",{
  users <- OCS$getUsers()
  expect_is(users, "character")
  expect_equal(length(users), 1L)
})

test_that("User Provisioning API - getUser",{
  user <- OCS$getUser("admin")
  expect_is(user, "list")
  user.df <- OCS$getUser("admin", TRUE)
  expect_is(user.df, "data.frame")
})

test_that("User Provisioning API - addUser",{
  added <- OCS$addUser("john.doe", password = "ocs4johnsecret")
  expect_true(added)
  users <- OCS$getUsers()
  expect_equal(length(users), 2L)
  user <- OCS$getUser("john.doe")
  expect_is(user, "list")
  expect_true(user$enabled)
  expect_equal(user$displayname, "john.doe")
  expect_null(user$email)
})

test_that("User Provisioning API - disableUser / enableUser",{
  disabled <- OCS$disableUser("john.doe")
  expect_true(disabled)
  user <- OCS$getUser("john.doe")
  expect_false(user$enabled)
  enabled <- OCS$enableUser("john.doe")
  expect_true(enabled)
  user <- OCS$getUser("john.doe")
  expect_true(user$enabled)
})

test_that("User Provisioning API - editUser",{
  edited <- OCS$editUser("john.doe", key = "display", value = "John Doe")
  expect_true(edited)
  john <- OCS$getUser("john.doe")
  expect_equal(john$displayname, "John Doe")
  edited2 <- OCS$editUserDisplayName("john.doe", displayName = "John Doe Jr.")
  expect_true(edited2)
  john <- OCS$getUser("john.doe")
  expect_equal(john$displayname, "John Doe Jr.")
})

test_that("User Provisioning API - getUserGroups",{
  admingroups <- OCS$getUserGroups("admin")
  expect_equal(admingroups, "admin")
})
  
test_that("User Provisioning API - getGroups",{
  groups <- OCS$getGroups()
  expect_equal(groups, "admin")
})

test_that("User Provisioning API - addGroup",{
  added <- OCS$addGroup("scientists")
  expect_true(added)
  groups <-  OCS$getGroups()
  expect_equal(groups, c("admin", "scientists"))
})

test_that("User Provisioning API - getGroup",{
  admin_group <- OCS$getGroup("admin")
  expect_equal(admin_group$id, "admin")
  expect_equal(admin_group$users, "admin")
  sc_group <- OCS$getGroup("scientists")
  expect_equal(sc_group$id, "scientists")
  expect_null(sc_group$users)
})

test_that("User Provisioning API - addToGroup",{
  added <- OCS$addToGroup("john.doe", "scientists")
  expect_true(added)
  expect_true("john.doe" %in% OCS$getGroup("scientists")$users)
})

test_that("User Provisioning API - removeFromGroup",{
  removed <- OCS$removeFromGroup("john.doe", "scientists")
  expect_true(removed)
  expect_false("john.doe" %in% OCS$getGroup("scientists")$users)
})

test_that("User Provisioning API - deleteUser",{
  deleted <- OCS$deleteUser("john.doe")
  expect_true(deleted)
  users <- OCS$getUsers()
  expect_false("john.doe" %in% users)
})

test_that("UserProvisioning API - deleteGroup",{
  deleted <- OCS$deleteGroup("scientists")
  expect_true(deleted)
  groups <- OCS$getGroups()
  expect_false("scientists" %in% groups)
})

#' ocsApiSharingManager
#' @docType class
#' @export
#' @keywords ocs manager sharing api
#' @return Object of \code{\link[R6]{R6Class}} for modelling an ocsManager for the Sharing API
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#' 
ocsApiSharingManager <-  R6Class("ocsApiSharingManager",
  inherit = ocsManager,
  public = list(
    
    #'@description Initialize manager
    #'@param url url
    #'@param user user
    #'@param pwd pwd
    #'@param logger logger
    #'@param keyring_backend backend to use with \pkg{keyring}. Default is \code{NULL}
    initialize = function(url, user, pwd, logger = NULL,
                          keyring_backend = NULL){
      super$initialize(url, user, pwd, logger, keyring_backend)
    },
    
    #OCS SHARING API
    #-------------------------------------------------------------------------------------------
    
    #'@description Get list of shares as \code{list}. To return as \code{data.frame}, 
    #'set \code{pretty = TRUE}. The method accepts additional parameters. 
    #'@param path path
    #'@param reshares reshares
    #'@param shared_with_me list only those shared with me?
    #'@param state state
    #'@param subfiles subfiles
    #'@param pretty pretty
    #'@return the list of shares as \code{list} or \code{data.frame}
    getShares = function(path = NULL, reshares = FALSE, shared_with_me = NULL,
                         state = NULL, subfiles = FALSE, 
                         pretty = FALSE){
      
      private$checkSharingAPIAvailability()
      
      allowedStates <- c("accepted", "all", "declined", "pending", "rejected")
      if(!is.null(state)) if(!(state %in% allowedStates)){
        errMsg <- sprintf("'state' should be one value among [%s]", 
                          paste(allowedStates, collapse=","))
        self$ERROR(errMsg)
        stop(errMsg)
      }
      
      request <- "ocs/v1.php/apps/files_sharing/api/v1/shares"
      get_req <- ocsRequest$new(
        type = "HTTP_GET", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        namedParams = list(
          path = path,
          reshares = reshares,
          shared_with_me = shared_with_me,
          state = state,
          subfiles = subfiles
        ),
        logger = self$loggerType
      )
      get_req$execute()
      get_resp <- get_req$getResponse()
      names(get_resp)
      get_out <- get_resp$ocs$data
      if(!is.null(get_out)) if(pretty){
        get_out <- as.data.frame(do.call("rbind",lapply(get_out, unlist)))
      }
      return(get_out)
    },
    
    #'@description Creates a share for the path (file or folder), given a name. The \code{shareType} should be among
    #'    values 'user','group','publiclink' or 'federated'.The \code{shareWith} is required for \code{shareType} 
    #'    'user' and 'group' and corresponds to the username or groupname. The \code{permissions} can be set among
    #'    values 'read', 'update', 'create', 'delete', 'read-write', 'share', 'all'. By default the permissions will 
    #'    be the default permissions set by the ocs server (by default 'all').
    #'@param path path
    #'@param name name
    #'@param shareType the type of share
    #'@param shareWith a list of users to share with
    #'@param publicUpload public upload
    #'@param password to set to access the share
    #'@param permissions permissions
    #'@param expireDate expire date
    createShare = function(path, name, shareType, shareWith, publicUpload = NULL, password = NULL, 
                           permissions = NULL, expireDate = NULL){
      
      private$checkSharingAPIAvailability()
      
      if(!private$capabilities$files_sharing$can_share){
        errMsg <- "The file sharing is not enabled!"
        self$ERROR(errMsg)
        stop(errMsg)
      }
      
      allowedShareTypes <- c("user", "group", "publiclink", "federated")
      if(!(shareType %in% allowedShareTypes)){
        errMsg <- sprintf("Share Type '%s' is not among allowed share types [%s]", 
                          shareType, paste0(allowedShareTypes, collapse=","))
        self$ERROR(errMsg)
        stop(errMsg)
      }
      shareType <- switch(shareType,
        "user" = 0,
        "group" = 1,
        "publiclink" = 3,
        "federated" = 6
      )
      
      if(!is.null(permissions)){
        allowedPermissions <- c("read", "update", "create", "delete", "read/write", "share", "all")
        if(!(permissions %in% allowedPermissions)){
          errMsg <- sprintf("Permission '%s' is not among allowed permissions [%s]",
                            permissions, paste0(allowedPermissions, collapse=","))
          self$ERROR(errMsg)
          stop(errMsg)
        }
        permissions <- switch(permissions,
          "read" = 1,
          "update" = 2,
          "create" = 4,
          "delete" = 8,
          "read/write" = 15,
          "share" = 16,
          "all" = 31
        )
      }else{
        permissions <- private$capabilities$files_sharing$default_permissions
      }
      
      if(!is.null(expireDate)){
        if(!is(expireDate, "Date")){
          errMsg <- "The 'expireDate' should be an object of class 'Date'"
          self$ERROR(errMsg)
          stop(errMsg)
        }else{
          expireDate <- as(expireDate, "character")
        }
      }
      
      if(!is.null(password)) if(!private$capabilities$files_sharing$public$password$enforced){
        self$WARN("The argument 'password' is ignored because OFCS Sharing public API 'password' is not enforced")
        password <- NULL
      }
      if(!is.null(expireDate)) if(!private$capabilities$files_sharing$public$expire_date$enabled){
        self$WARN("The argument 'expireDate' is ignored because OCS Sharing public API 'expireDate' is not enabled")
        expireDate <- NULL
      }
      
      request <- "ocs/v1.php/apps/files_sharing/api/v1/shares"
      post_req <- ocsRequest$new(
        type = "HTTP_POST", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        namedParams = list(
          name = name,
          path = path,
          shareType = shareType,
          shareWith = shareWith,
          publicUpload = publicUpload,
          password = password,
          permissions = permissions,
          expireDate = expireDate
        ),
        logger = self$loggerType
      )
      post_req$execute()
      return(post_req$getResponse())
    },
    
    #'@description Shares a resource (file or folder) with a user given its username handled with argument \code{username}. 
    #'   The \code{permissions} can be set among values 'read', 'update', 'create', 'delete', 'read-write', 'share', 
    #'   'all'. By default the permissions will be the default permissions set by the ocs server (by default 'all').
    #'   Returns 
    #'@param path path
    #'@param name name
    #'@param username username
    #'@param group group
    #'@param permissions permissions
    #'@param pretty pretty
    #'@return the share properties as \code{list} (or as\code{data.frame} if \code{pretty} is set to TRUE).
    shareWithUser = function(path, name, username, permissions = NULL, pretty = FALSE){
      share_resp <- self$createShare(
        path = path,
        name = URLencode(name),
        shareType = "user",
        shareWith = username,
        permissions = permissions
      )
      share_out <- share_resp$ocs$data
      if(!is.null(share_out)) if(pretty){
        share_out <- as.data.frame(t(share_out))
      }
      return(share_out)
    },
    
    #'@description Shares a resource (file or folder) with a group given its name handled with argument \code{group}. 
    #'   The \code{permissions} can be set among values 'read', 'update', 'create', 'delete', 'read-write', 'share', 
    #'   'all'. By default the permissions will be the default permissions set by the ocs server (by default 'all').
    #'@param path path
    #'@param name name
    #'@param group group
    #'@param permissions permissions
    #'@param pretty pretty
    #'@return the share properties as \code{list} (or as\code{data.frame} if \code{pretty} is set to TRUE).
    shareWithGroup = function(path, name, group, permissions = NULL, pretty = FALSE){
      share_resp <- self$createShare(
        path = path,
        name = URLencode(name),
        shareType = "group",
        shareWith = group,
        permissions = permissions
      )
      share_out <- share_resp$ocs$data
      if(!is.null(share_out)) if(pretty){
        share_out <- as.data.frame(t(share_out))
      }
      return(share_out)
    },
    
    #'@description Shares a resource (file or folder) as public link. The function returns the public link generated by ocs.
    #'@param path path
    #'@param name name
    #'@param publicUpload public upload?
    #'@param password password
    #'@param permissions permissions
    #'@param expireDate expire date
    #'@param return the public sharing link
    shareAsPublicLink = function(path, name = NULL, publicUpload = FALSE, password = NULL, 
                                 permissions = NULL, expireDate = NULL){
      if(!private$capabilities$files_sharing$public$enabled){
        errMsg <- "The OCS Sharing public API is not enabled. Impossible to share as public link!"
        self$ERROR(errMsg)
        stop(errMsg)
      }
      if(is.null(name)) name <- private$capabilities$files_sharing$public$defaultPublicLinkShareName
      
      sharelink_resp <- self$createShare(
        path = path,
        name = URLencode(name),
        shareType = "publiclink",
        shareWith = NULL,
        publicUpload = publicUpload,
        password = password,
        permissions = permissions,
        expireDate = expireDate
      )
      self$INFO(sprintf("Public link for '%s': %s", path, sharelink_resp$ocs$data$url))
      return(sharelink_resp$ocs$data$url)
    }
  )
)

#' ocsApiWebdavManager
#' @docType class
#' @export
#' @keywords ocs manager webdav api
#' @return Object of \code{\link[R6]{R6Class}} for modelling an ocsManager for Webdav API
#' @format \code{\link[R6]{R6Class}} object.
#' 
#' @author Emmanuel Blondel <emmanuel.blondel1@@gmail.com>
#' 
ocsApiWebdavManager <-  R6Class("ocsApiWebdavManager",
  inherit = ocsManager,
  public = list(
    
    #'@description Initialize manager
    #'@param url url
    #'@param user user
    #'@param pwd pwd
    #'@param logger logger
    #'@param keyring_backend backend to use with \pkg{keyring}. Default is \code{NULL}
    initialize = function(url, user, pwd, logger = NULL,
                          keyring_backend = NULL){
      super$initialize(url, user, pwd, logger, keyring_backend)
    },
    
    #OCS WEBDAV API
    #-------------------------------------------------------------------------------------------
    
    #'@description Get the 'ocs' WebDAV root URL
    getWebdavRoot = function(){
      return(private$capabilities$core[["webdav-root"]])
    },
    
    #'@description WebDAV method to list folders/files given a relative path. The relative path is set
    #'    to \code{"/"} by default, which corresponds to the root of the 'ocs' repository.
    #'@param relPath relative path
    #'@return the list of files
    listFiles = function(relPath = "/"){
      if(!startsWith(relPath, "/")) relPath <- paste0("/", relPath)
      request <- paste0(self$getWebdavRoot(), relPath)
      list_req <- ocsRequest$new(
        type = "WEBDAV_PROPFIND", private$url, URLencode(request),
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        logger = self$loggerType
      )
      list_req$execute()
      list_resp <- list_req$getResponse()
      return(list_resp)
    },
    
    #'@description WebDAV method to make a collection. By default \code{relPath} is set to \code{"/"} (root).
    #'    The \code{name} is the name of the new collection to be created. The function is recursive
    #'    in the sense that a \code{name} can be provided as relative path of a collection tree 
    #'    (eg \code{newfolder1/newfolder2/newfolder3}), the function will create recursively as 
    #'    many collections are handled in the name. 
    #'@param name name
    #'@param relPath relative path
    makeCollection = function(name, relPath = "/"){
      col_names <- unlist(strsplit(name, "/"))
      if(!startsWith(relPath, "/")) relPath <- paste0("/", relPath)
      if(!endsWith(relPath, "/")) relPath <- paste0(relPath, "/")
      if(length(col_names)==1){
        request <- URLencode(paste0(self$getWebdavRoot(), relPath, name))
        mkcol_req <- ocsRequest$new(
          type = "WEBDAV_MKCOL", private$url, request,
          private$user, pwd = private$getPassword(), 
          token = private$getToken(), cookies = private$cookies,
          logger = self$loggerType
        )
        mkcol_req$execute()
        mkcol_resp <- mkcol_req$getResponse()
        return(mkcol_resp)
      }else{
        self$INFO(sprintf("Nested collections detected in '%s'. Splitting name to make nested collections", name))
        for(i in 1:length(col_names)){
          newRelPath <- relPath
          if(i>1) newRelPath <- paste0(newRelPath, paste(col_names[1:(i-1)], collapse="/"), "/")
          print(col_names[i])
          print(newRelPath)
          if(!paste0(col_names[i],"/") %in% self$listFiles(relPath = newRelPath)$name) {
            self$makeCollection(col_names[i], newRelPath)
          }
        }
      }
    },
    
    #'@description WebDAV method to upload a file. By default \code{relPath} is set to \code{"/"} (root).
    #'@param filename file name
    #'@param relPath relative path
    #'@param delete_if_existing delete if existing file? Default is \code{FALSE}
    uploadFile = function(filename, relPath = "/", delete_if_existing = FALSE){
      
      if(delete_if_existing){
        try(self$deleteFile(filename = filename, relPath = relPath))
      }
      
      if(!startsWith(relPath, "/")) relPath <- paste0("/", relPath)
      if(!endsWith(relPath, "/")) relPath <- paste0(relPath, "/")
      request <- URLencode(paste0(self$getWebdavRoot(), relPath, basename(filename)))
      self$INFO(sprintf("WEBDAV - Uploading file '%s' at '%s'", 
                        filename, paste(private$url, request, sep="/")))
      upload_req <- ocsRequest$new(
        type = "HTTP_PUT", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        filename = filename,
        logger = self$loggerType
      )
      upload_req$execute()
      if(upload_req$getStatus() %in% c(201, 204)){
        self$INFO(sprintf("Successfuly uploaded file '%s' at '%s'",
                          filename, paste(private$url, request, sep="/")))
      }else{
        errMsg <- sprintf("WEBDAV - Error while uploading '%s' at '%s'",
                          filename, paste(private$url, request, sep="/"))
        self$ERROR(errMsg)
        stop(errMsg)
      }
      upload_resp <- upload_req$getResponse()
      return(upload_resp)
    },
    
    #'@description WebDAV method to delete a file. By default \code{relPath} is set to \code{"/"} (root).
    #'@param filename file name
    #'@param relPath relative path
    deleteFile = function(filename, relPath = "/"){
      if(!startsWith(relPath, "/")) relPath <- paste0("/", relPath)
      if(!endsWith(relPath, "/")) relPath <- paste0(relPath, "/")
      request <- URLencode(paste0(self$getWebdavRoot(), relPath, basename(filename)))
      self$INFO(sprintf("WEBDAV - Delete file '%s' at '%s'", 
                        filename, paste(private$url, request, sep="/")))
      upload_req <- ocsRequest$new(
        type = "HTTP_DELETE", private$url, request,
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        filename = filename,
        logger = self$loggerType
      )
      upload_req$execute()
      if(upload_req$getStatus()==204){
        self$INFO(sprintf("Successfuly deleted file '%s' at '%s'",
                          filename, paste(private$url, request, sep="/")))
      }else{
        errMsg <- sprintf("WEBDAV - Error while deleting '%s' at '%s'",
                          filename, paste(private$url, request, sep="/"))
        self$ERROR(errMsg)
        stop(errMsg)
      }
      upload_resp <- upload_req$getResponse()
      return(upload_resp)
    },
    
    #'@description Downloads a file
    #'@param relPath relative path
    #'@param filename file name
    #'@param outdir the out directory where to download the file
    downloadFile = function(relPath, filename, outdir = "."){
      request <- URLencode(sprintf("remote.php/dav/files/%s/%s/%s", private$user, relPath, filename))
      file_req <- ocsRequest$new(
        type = "HTTP_GET", private$url, request, format = NULL, namedParams = list(),
        private$user, pwd = private$getPassword(), 
        token = private$getToken(), cookies = private$cookies,
        logger = self$loggerType
      )
      file_req$execute()
      file_resp <- file_req$getResponse()
      writeBin(object = file_resp, con = file.path(outdir, filename))
      return(file.path(outdir, filename))
    },
    
    #'@description Get details of a shared public file given its share token
    #'@param share_token the share token
    getPublicFile = function(share_token){
      request <- file.path("remote.php/dav/public-files", share_token)
      file_req <- ocsRequest$new(
        type = "WEBDAV_PROPFIND", private$url, request,
        anonymous = TRUE, logger = self$loggerType
      )
      file_req$execute()
      file_resp <- file_req$getResponse()
      return(file_resp)
    }
    
  )
)

# test_api_webdav.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for Webdav API methods
#=======================
require(ocs4R, quietly = TRUE)
require(testthat)

context("api-webdav")

test_that("WEBDAV API - list files",{
  files <- OCS$listFiles()
  expect_is(files, "data.frame")
  expect_true(nrow(files)>0)
  subfiles <- OCS$listFiles("Documents")
  expect_is(subfiles, "data.frame")
  expect_true(nrow(subfiles)>0)
})

test_that("WEBDAV API - make collection",{
  myfolder_url <- OCS$makeCollection("myfolder")
  expect_is(myfolder_url, "character")
  expect_true(endsWith(myfolder_url, "myfolder"))
  files <- OCS$listFiles()
  expect_true("myfolder/" %in% files$name)
  subfiles <- OCS$listFiles("myfolder")
  expect_is(subfiles, "data.frame")
  expect_true(nrow(subfiles)==0)
})

test_that("WEBDAV API - upload file - from working dir",{
  nf <- file.create("test.txt")
  nfcon <- file("test.txt", "wb")
  writeChar("This is a test file", nfcon)
  close(nfcon)
  #1st upload
  nfname <- OCS$uploadFile("test.txt")
  expect_true("test.txt" %in% OCS$listFiles()$name)
  #2d upload (overwrite test)
  nfname <- OCS$uploadFile("test.txt")
  expect_true("test.txt" %in% OCS$listFiles()$name)
})

test_that("WEBDAV API - upload file - from subdir",{
  dir.create("myfolder")
  nf <- file.create(file.path("myfolder","test.txt"))
  nfcon <- file(file.path("myfolder","test.txt"), "wb")
  writeChar("This is a test file", nfcon)
  close(nfcon)
  nfname <- OCS$uploadFile("myfolder/test.txt", relPath = "myfolder")
  expect_true("test.txt" %in% OCS$listFiles("myfolder")$name)
})

library(testthat)
library(ocs4R)

if(Sys.getenv("NOT_CRAN") == "true"){

  #test environment
  ocs_url <- "http://localhost:8080"
  ocs_user <- "admin"
  ocs_pwd <- "admin"
  
  OCS <- ocsManager$new(
    url = ocs_url, 
    user = ocs_user, 
    pwd = ocs_pwd, 
    logger = "DEBUG"
  )
  
  if(is(OCS, "ocsManager")){
    cat(sprintf("OCS REST endpoint at '%s' configured with token. Running integration tests...\n", ocs_url))
    test_check("ocs4R")
  }else{
    cat("OCS REST endpoint at '%s' not configured. Skipping integration tests...\n")
  }

}

# test_api_sharing.R
# Author: Emmanuel Blondel <emmanuel.blondel1@gmail.com>
#
# Description: Unit tests for Sharing API methods
#=======================
require(ocs4R, quietly = TRUE)
require(testthat)

context("api-sharing")

test_that("SHARING API - share",{
 
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ocsApiUserProvisioningManager.R
\docType{class}
\name{ocsApiUserProvisioningManager}
\alias{ocsApiUserProvisioningManager}
\title{ocsApiUserProvisioningManager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling an ocsManager for Webdav API
}
\description{
ocsApiUserProvisioningManager

ocsApiUserProvisioningManager
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{manager}
\keyword{ocs}
\keyword{userprovisioning}
\section{Super classes}{
\code{\link[ocs4R:ocs4RLogger]{ocs4R::ocs4RLogger}} -> \code{\link[ocs4R:ocsManager]{ocs4R::ocsManager}} -> \code{ocsApiUserProvisioningManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-ocsApiUserProvisioningManager-new}{\code{ocsApiUserProvisioningManager$new()}}
\item \href{#method-ocsApiUserProvisioningManager-addUser}{\code{ocsApiUserProvisioningManager$addUser()}}
\item \href{#method-ocsApiUserProvisioningManager-getUsers}{\code{ocsApiUserProvisioningManager$getUsers()}}
\item \href{#method-ocsApiUserProvisioningManager-getUser}{\code{ocsApiUserProvisioningManager$getUser()}}
\item \href{#method-ocsApiUserProvisioningManager-editUser}{\code{ocsApiUserProvisioningManager$editUser()}}
\item \href{#method-ocsApiUserProvisioningManager-editUserDisplayName}{\code{ocsApiUserProvisioningManager$editUserDisplayName()}}
\item \href{#method-ocsApiUserProvisioningManager-editUserEmail}{\code{ocsApiUserProvisioningManager$editUserEmail()}}
\item \href{#method-ocsApiUserProvisioningManager-editUserPassword}{\code{ocsApiUserProvisioningManager$editUserPassword()}}
\item \href{#method-ocsApiUserProvisioningManager-editUserQuota}{\code{ocsApiUserProvisioningManager$editUserQuota()}}
\item \href{#method-ocsApiUserProvisioningManager-enableUser}{\code{ocsApiUserProvisioningManager$enableUser()}}
\item \href{#method-ocsApiUserProvisioningManager-disableUser}{\code{ocsApiUserProvisioningManager$disableUser()}}
\item \href{#method-ocsApiUserProvisioningManager-deleteUser}{\code{ocsApiUserProvisioningManager$deleteUser()}}
\item \href{#method-ocsApiUserProvisioningManager-getUserGroups}{\code{ocsApiUserProvisioningManager$getUserGroups()}}
\item \href{#method-ocsApiUserProvisioningManager-addToGroup}{\code{ocsApiUserProvisioningManager$addToGroup()}}
\item \href{#method-ocsApiUserProvisioningManager-removeFromGroup}{\code{ocsApiUserProvisioningManager$removeFromGroup()}}
\item \href{#method-ocsApiUserProvisioningManager-createSubadmin}{\code{ocsApiUserProvisioningManager$createSubadmin()}}
\item \href{#method-ocsApiUserProvisioningManager-removeSubadmin}{\code{ocsApiUserProvisioningManager$removeSubadmin()}}
\item \href{#method-ocsApiUserProvisioningManager-getSubadminGroups}{\code{ocsApiUserProvisioningManager$getSubadminGroups()}}
\item \href{#method-ocsApiUserProvisioningManager-getGroups}{\code{ocsApiUserProvisioningManager$getGroups()}}
\item \href{#method-ocsApiUserProvisioningManager-addGroup}{\code{ocsApiUserProvisioningManager$addGroup()}}
\item \href{#method-ocsApiUserProvisioningManager-getGroup}{\code{ocsApiUserProvisioningManager$getGroup()}}
\item \href{#method-ocsApiUserProvisioningManager-deleteGroup}{\code{ocsApiUserProvisioningManager$deleteGroup()}}
\item \href{#method-ocsApiUserProvisioningManager-getSubadmins}{\code{ocsApiUserProvisioningManager$getSubadmins()}}
\item \href{#method-ocsApiUserProvisioningManager-clone}{\code{ocsApiUserProvisioningManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="ERROR"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-ERROR'><code>ocs4R::ocs4RLogger$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="INFO"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-INFO'><code>ocs4R::ocs4RLogger$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="WARN"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-WARN'><code>ocs4R::ocs4RLogger$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="getClass"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-getClass'><code>ocs4R::ocs4RLogger$getClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="getClassName"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-getClassName'><code>ocs4R::ocs4RLogger$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="logger"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-logger'><code>ocs4R::ocs4RLogger$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="connect"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-connect'><code>ocs4R::ocsManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getAPISharingManager"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getAPISharingManager'><code>ocs4R::ocsManager$getAPISharingManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getAPIUserProvisioningManager"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getAPIUserProvisioningManager'><code>ocs4R::ocsManager$getAPIUserProvisioningManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getAPIWebdavManager"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getAPIWebdavManager'><code>ocs4R::ocsManager$getAPIWebdavManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getCapabilities"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getCapabilities'><code>ocs4R::ocsManager$getCapabilities()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getVersion"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getVersion'><code>ocs4R::ocsManager$getVersion()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-new"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-new}{}}}
\subsection{Method \code{new()}}{
Initialize manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$new(
  url,
  user,
  pwd,
  logger = NULL,
  keyring_backend = "env"
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{url}}{url}

\item{\code{user}}{user}

\item{\code{pwd}}{pwd}

\item{\code{logger}}{logger}

\item{\code{keyring_backend}}{backend to use with \pkg{keyring}. Default is \code{NULL}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-addUser"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-addUser}{}}}
\subsection{Method \code{addUser()}}{
Adds a user given a \code{userid} (required). All other fields (email, password, groups) are
   optional for the user creation. Returns \code{TRUE} if the user is added, \code{FALSE} otherwise.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$addUser(
  userid,
  email = NULL,
  password = NULL,
  groups = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}

\item{\code{email}}{email}

\item{\code{password}}{user password}

\item{\code{groups}}{groups}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-getUsers"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-getUsers}{}}}
\subsection{Method \code{getUsers()}}{
Get the list of users. This method returns a vector of class 'character' giving 
the user IDs available in the OCS cloud plateform.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$getUsers()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-getUser"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-getUser}{}}}
\subsection{Method \code{getUser()}}{
Get the user details from its \code{userid}. If the argument \code{pretty} is set to TRUE,
   this will return an object of class \code{data.frame}, otherwise (by default) it returns 
   an object of class \code{list}.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$getUser(userid, pretty = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}

\item{\code{pretty}}{pretty}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-editUser"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-editUser}{}}}
\subsection{Method \code{editUser()}}{
Edits a user, identifier by a userid. The user property to be edited should be set using its
   key (eg display) and the value to be modified for this key. Returns \code{TRUE} if the user 
   is edited, \code{FALSE} otherwise.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$editUser(userid, key, value)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}

\item{\code{key}}{key}

\item{\code{value}}{value}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-editUserDisplayName"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-editUserDisplayName}{}}}
\subsection{Method \code{editUserDisplayName()}}{
Edits a user display name.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$editUserDisplayName(userid, displayName)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}

\item{\code{displayName}}{display name}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-editUserEmail"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-editUserEmail}{}}}
\subsection{Method \code{editUserEmail()}}{
Edits a user email
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$editUserEmail(userid, email)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}

\item{\code{email}}{email}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-editUserPassword"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-editUserPassword}{}}}
\subsection{Method \code{editUserPassword()}}{
Edits a user password
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$editUserPassword(userid, password)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}

\item{\code{password}}{password}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-editUserQuota"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-editUserQuota}{}}}
\subsection{Method \code{editUserQuota()}}{
Edits a user quota
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$editUserQuota(userid, quota)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}

\item{\code{quota}}{quota}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-enableUser"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-enableUser}{}}}
\subsection{Method \code{enableUser()}}{
Enables a user
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$enableUser(userid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if enabled, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-disableUser"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-disableUser}{}}}
\subsection{Method \code{disableUser()}}{
Disables a user
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$disableUser(userid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if disabled, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-deleteUser"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-deleteUser}{}}}
\subsection{Method \code{deleteUser()}}{
Deletes a user
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$deleteUser(userid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-getUserGroups"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-getUserGroups}{}}}
\subsection{Method \code{getUserGroups()}}{
Get user groups
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$getUserGroups(userid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the user groups
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-addToGroup"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-addToGroup}{}}}
\subsection{Method \code{addToGroup()}}{
Adds a user to a group.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$addToGroup(userid, groupid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}

\item{\code{groupid}}{group ID}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-removeFromGroup"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-removeFromGroup}{}}}
\subsection{Method \code{removeFromGroup()}}{
Removes a user from a group.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$removeFromGroup(userid, groupid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{userid}}{user ID}

\item{\code{groupid}}{group ID}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if removed, \code{FALSE} otherwise
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-createSubadmin"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-createSubadmin}{}}}
\subsection{Method \code{createSubadmin()}}{
Creates a subadmin
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$createSubadmin()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-removeSubadmin"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-removeSubadmin}{}}}
\subsection{Method \code{removeSubadmin()}}{
Removes a subadmin
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$removeSubadmin()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-getSubadminGroups"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-getSubadminGroups}{}}}
\subsection{Method \code{getSubadminGroups()}}{
Get subadmin groups
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$getSubadminGroups()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-getGroups"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-getGroups}{}}}
\subsection{Method \code{getGroups()}}{
Get the list of groups. This method returns a vector of class 
'character' giving the usergroups IDs
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$getGroups(
  search = NULL,
  limit = NULL,
  offset = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{search}}{search}

\item{\code{limit}}{limit}

\item{\code{offset}}{offset}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-addGroup"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-addGroup}{}}}
\subsection{Method \code{addGroup()}}{
Adds a group
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$addGroup(groupid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{groupid}}{group ID}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if added, \code{FALSE}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-getGroup"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-getGroup}{}}}
\subsection{Method \code{getGroup()}}{
Gets a group
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$getGroup(groupid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{groupid}}{group ID}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the group as \code{list} including the group ID and the list of users
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-deleteGroup"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-deleteGroup}{}}}
\subsection{Method \code{deleteGroup()}}{
Deletes a group
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$deleteGroup(groupid)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{groupid}}{group ID}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
\code{TRUE} if deleted, \code{FALSE}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-getSubadmins"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-getSubadmins}{}}}
\subsection{Method \code{getSubadmins()}}{
Get subadmins
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$getSubadmins()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiUserProvisioningManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiUserProvisioningManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiUserProvisioningManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ocsRequest.R
\docType{class}
\name{ocsRequest}
\alias{ocsRequest}
\title{ocsRequest}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a generic 'ocs' web-service request
}
\description{
ocsRequest

ocsRequest
}
\note{
Abstract class used internally by \pkg{ocs4R}
}
\section{Methods}{

\describe{
 \item{\code{new(type, url, request, user, pwd, token, cookies,
                 format, namedParams, content, contentType, filename, logger, ...)}}{
   This method is used to instantiate a object for doing an 'ocs' web-service request
 }
 \item{\code{getRequest()}}{
   Get the request payload
 }
 \item{\code{getRequestHeaders()}}{
   Get the request headers
 }
 \item{\code{getStatus()}}{
   Get the request status code
 }
 \item{\code{getResponse()}}{
   Get the request response
 }
 \item{\code{getException()}}{
   Get the exception (in case of request failure)
 }
 \item{\code{getResult()}}{
   Get the result \code{TRUE} if the request is successful, \code{FALSE} otherwise
 }
}
}

\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{ocs}
\keyword{request}
\section{Super class}{
\code{\link[ocs4R:ocs4RLogger]{ocs4R::ocs4RLogger}} -> \code{ocsRequest}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-ocsRequest-new}{\code{ocsRequest$new()}}
\item \href{#method-ocsRequest-execute}{\code{ocsRequest$execute()}}
\item \href{#method-ocsRequest-getRequest}{\code{ocsRequest$getRequest()}}
\item \href{#method-ocsRequest-getRequestHeaders}{\code{ocsRequest$getRequestHeaders()}}
\item \href{#method-ocsRequest-getStatus}{\code{ocsRequest$getStatus()}}
\item \href{#method-ocsRequest-getResponse}{\code{ocsRequest$getResponse()}}
\item \href{#method-ocsRequest-getException}{\code{ocsRequest$getException()}}
\item \href{#method-ocsRequest-getResult}{\code{ocsRequest$getResult()}}
\item \href{#method-ocsRequest-setResult}{\code{ocsRequest$setResult()}}
\item \href{#method-ocsRequest-clone}{\code{ocsRequest$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="ERROR"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-ERROR'><code>ocs4R::ocs4RLogger$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="INFO"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-INFO'><code>ocs4R::ocs4RLogger$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="WARN"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-WARN'><code>ocs4R::ocs4RLogger$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="getClass"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-getClass'><code>ocs4R::ocs4RLogger$getClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="getClassName"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-getClassName'><code>ocs4R::ocs4RLogger$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="logger"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-logger'><code>ocs4R::ocs4RLogger$logger()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-new"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-new}{}}}
\subsection{Method \code{new()}}{
This method is used to instantiate a object for doing an 'ocs' web-service request
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$new(
  type,
  url,
  request,
  user = NULL,
  pwd = NULL,
  token = NULL,
  cookies = NULL,
  format = "json",
  namedParams = list(),
  content = NULL,
  contentType = "text/plain",
  filename = NULL,
  logger = NULL,
  ...
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{type}}{type of request}

\item{\code{url}}{url}

\item{\code{request}}{request}

\item{\code{user}}{user}

\item{\code{pwd}}{pwd}

\item{\code{token}}{token}

\item{\code{cookies}}{cookies}

\item{\code{format}}{format. Default is "json"}

\item{\code{namedParams}}{a list of named parameters}

\item{\code{content}}{content}

\item{\code{contentType}}{content type. Default is "text/plain"}

\item{\code{filename}}{file name}

\item{\code{logger}}{logger}

\item{\code{...}}{additional parameters}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-execute"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-execute}{}}}
\subsection{Method \code{execute()}}{
Executes the request
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$execute()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-getRequest"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-getRequest}{}}}
\subsection{Method \code{getRequest()}}{
Get request
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$getRequest()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-getRequestHeaders"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-getRequestHeaders}{}}}
\subsection{Method \code{getRequestHeaders()}}{
Get request headers
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$getRequestHeaders()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-getStatus"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-getStatus}{}}}
\subsection{Method \code{getStatus()}}{
Get status
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$getStatus()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-getResponse"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-getResponse}{}}}
\subsection{Method \code{getResponse()}}{
Get response
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$getResponse()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-getException"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-getException}{}}}
\subsection{Method \code{getException()}}{
Get exception
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$getException()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-getResult"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-getResult}{}}}
\subsection{Method \code{getResult()}}{
Get result
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$getResult()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-setResult"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-setResult}{}}}
\subsection{Method \code{setResult()}}{
Set result
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$setResult(result)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{result}}{result}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsRequest-clone"></a>}}
\if{latex}{\out{\hypertarget{method-ocsRequest-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsRequest$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ocsApiSharingManager.R
\docType{class}
\name{ocsApiSharingManager}
\alias{ocsApiSharingManager}
\title{ocsApiSharingManager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling an ocsManager for the Sharing API
}
\description{
ocsApiSharingManager

ocsApiSharingManager
}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{api}
\keyword{manager}
\keyword{ocs}
\keyword{sharing}
\section{Super classes}{
\code{\link[ocs4R:ocs4RLogger]{ocs4R::ocs4RLogger}} -> \code{\link[ocs4R:ocsManager]{ocs4R::ocsManager}} -> \code{ocsApiSharingManager}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-ocsApiSharingManager-new}{\code{ocsApiSharingManager$new()}}
\item \href{#method-ocsApiSharingManager-getShares}{\code{ocsApiSharingManager$getShares()}}
\item \href{#method-ocsApiSharingManager-createShare}{\code{ocsApiSharingManager$createShare()}}
\item \href{#method-ocsApiSharingManager-shareWithUser}{\code{ocsApiSharingManager$shareWithUser()}}
\item \href{#method-ocsApiSharingManager-shareWithGroup}{\code{ocsApiSharingManager$shareWithGroup()}}
\item \href{#method-ocsApiSharingManager-shareAsPublicLink}{\code{ocsApiSharingManager$shareAsPublicLink()}}
\item \href{#method-ocsApiSharingManager-clone}{\code{ocsApiSharingManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="ERROR"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-ERROR'><code>ocs4R::ocs4RLogger$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="INFO"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-INFO'><code>ocs4R::ocs4RLogger$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="WARN"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-WARN'><code>ocs4R::ocs4RLogger$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="getClass"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-getClass'><code>ocs4R::ocs4RLogger$getClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="getClassName"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-getClassName'><code>ocs4R::ocs4RLogger$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="logger"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-logger'><code>ocs4R::ocs4RLogger$logger()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="connect"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-connect'><code>ocs4R::ocsManager$connect()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getAPISharingManager"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getAPISharingManager'><code>ocs4R::ocsManager$getAPISharingManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getAPIUserProvisioningManager"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getAPIUserProvisioningManager'><code>ocs4R::ocsManager$getAPIUserProvisioningManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getAPIWebdavManager"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getAPIWebdavManager'><code>ocs4R::ocsManager$getAPIWebdavManager()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getCapabilities"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getCapabilities'><code>ocs4R::ocsManager$getCapabilities()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocsManager" data-id="getVersion"><a href='../../ocs4R/html/ocsManager.html#method-ocsManager-getVersion'><code>ocs4R::ocsManager$getVersion()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiSharingManager-new"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiSharingManager-new}{}}}
\subsection{Method \code{new()}}{
Initialize manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiSharingManager$new(url, user, pwd, logger = NULL, keyring_backend = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{url}}{url}

\item{\code{user}}{user}

\item{\code{pwd}}{pwd}

\item{\code{logger}}{logger}

\item{\code{keyring_backend}}{backend to use with \pkg{keyring}. Default is \code{NULL}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiSharingManager-getShares"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiSharingManager-getShares}{}}}
\subsection{Method \code{getShares()}}{
Get list of shares as \code{list}. To return as \code{data.frame}, 
set \code{pretty = TRUE}. The method accepts additional parameters.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiSharingManager$getShares(
  path = NULL,
  reshares = FALSE,
  shared_with_me = NULL,
  state = NULL,
  subfiles = FALSE,
  pretty = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{path}}{path}

\item{\code{reshares}}{reshares}

\item{\code{shared_with_me}}{list only those shared with me?}

\item{\code{state}}{state}

\item{\code{subfiles}}{subfiles}

\item{\code{pretty}}{pretty}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the list of shares as \code{list} or \code{data.frame}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiSharingManager-createShare"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiSharingManager-createShare}{}}}
\subsection{Method \code{createShare()}}{
Creates a share for the path (file or folder), given a name. The \code{shareType} should be among
   values 'user','group','publiclink' or 'federated'.The \code{shareWith} is required for \code{shareType} 
   'user' and 'group' and corresponds to the username or groupname. The \code{permissions} can be set among
   values 'read', 'update', 'create', 'delete', 'read-write', 'share', 'all'. By default the permissions will 
   be the default permissions set by the ocs server (by default 'all').
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiSharingManager$createShare(
  path,
  name,
  shareType,
  shareWith,
  publicUpload = NULL,
  password = NULL,
  permissions = NULL,
  expireDate = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{path}}{path}

\item{\code{name}}{name}

\item{\code{shareType}}{the type of share}

\item{\code{shareWith}}{a list of users to share with}

\item{\code{publicUpload}}{public upload}

\item{\code{password}}{to set to access the share}

\item{\code{permissions}}{permissions}

\item{\code{expireDate}}{expire date}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiSharingManager-shareWithUser"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiSharingManager-shareWithUser}{}}}
\subsection{Method \code{shareWithUser()}}{
Shares a resource (file or folder) with a user given its username handled with argument \code{username}. 
  The \code{permissions} can be set among values 'read', 'update', 'create', 'delete', 'read-write', 'share', 
  'all'. By default the permissions will be the default permissions set by the ocs server (by default 'all').
  Returns
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiSharingManager$shareWithUser(
  path,
  name,
  username,
  permissions = NULL,
  pretty = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{path}}{path}

\item{\code{name}}{name}

\item{\code{username}}{username}

\item{\code{permissions}}{permissions}

\item{\code{pretty}}{pretty}

\item{\code{group}}{group}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the share properties as \code{list} (or as\code{data.frame} if \code{pretty} is set to TRUE).
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiSharingManager-shareWithGroup"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiSharingManager-shareWithGroup}{}}}
\subsection{Method \code{shareWithGroup()}}{
Shares a resource (file or folder) with a group given its name handled with argument \code{group}. 
  The \code{permissions} can be set among values 'read', 'update', 'create', 'delete', 'read-write', 'share', 
  'all'. By default the permissions will be the default permissions set by the ocs server (by default 'all').
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiSharingManager$shareWithGroup(
  path,
  name,
  group,
  permissions = NULL,
  pretty = FALSE
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{path}}{path}

\item{\code{name}}{name}

\item{\code{group}}{group}

\item{\code{permissions}}{permissions}

\item{\code{pretty}}{pretty}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
the share properties as \code{list} (or as\code{data.frame} if \code{pretty} is set to TRUE).
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiSharingManager-shareAsPublicLink"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiSharingManager-shareAsPublicLink}{}}}
\subsection{Method \code{shareAsPublicLink()}}{
Shares a resource (file or folder) as public link. The function returns the public link generated by ocs.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiSharingManager$shareAsPublicLink(
  path,
  name = NULL,
  publicUpload = FALSE,
  password = NULL,
  permissions = NULL,
  expireDate = NULL
)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{path}}{path}

\item{\code{name}}{name}

\item{\code{publicUpload}}{public upload?}

\item{\code{password}}{password}

\item{\code{permissions}}{permissions}

\item{\code{expireDate}}{expire date}

\item{\code{return}}{the public sharing link}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsApiSharingManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-ocsApiSharingManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsApiSharingManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ocsManager.R
\docType{class}
\name{ocsManager}
\alias{ocsManager}
\title{ocsManager}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling an ocsManager
}
\description{
ocsManager

ocsManager
}
\examples{
\dontrun{
   #Not Run:
   #Connect to an OCS API
   OCS <- ocsManager$new(url = ocs_url, user = ocs_user, pwd = ocs_pwd, logger = "DEBUG")
   version <- OCS$getVersion()
   caps <- OCS$getCapabilities()
   
   #OCS User Provisioning API
   #-------------------------------------
   #users
   users <- OCS$getUsers() #get users
   user <- OCS$getUser("admin") #get a user
   user.df <- OCS$getUser("admin", TRUE) #the same user as data.frame
   added <- OCS$addUser("john.doe", password = "ocs4john") #add a user
   disabled <- OCS$disableUser("john.doe") #disable a user
   enabled <- OCS$enableUser("john.doe") #enable auser
   edited <- OCS$editUser("john.doe", key = "display", value = "John Doe") #edit user
   #edit some user field
   edited2 <- OCS$editUserDisplayName("john.doe", displayName = "John Doe Jr.") 
   deleted <- OCS$deleteUser("john.doe")
   
   #groups
   admingroups <- OCS$getUserGroups("admin")
   groups <- OCS$getGroups()
   added <- OCS$addGroup("scientists") #add a new group
   sc_group <- OCS$getGroup("scientists") #get group details
   added <- OCS$addToGroup("john.doe", "scientists") #add user to group
   removed <- OCS$removeFromGroup("john.doe", "scientists") #remove user from group
   deleted <- OCS$deleteGroup("scientists")
   
   #OCS Webdav API
   #-------------------------------------
   #list files
   files <- OCS$listFiles()
   subfiles <- OCS$listFiles("Documents")
   #make collection
   OCS$makeCollection("myfolder")
   subfiles <- OCS$listFiles("myfolder")
   #upload a file?
   filename <- "magic.txt"
   file.create(filename); writeLines("ocs4R is great", filename)
   #we upload the file in 'Documents' folder
   OCS$uploadFile(filename, "/Documents")
   #check if file is uploaded
   OCS$listFiles('Documents')
   
   #OCS Sharing API
   #-------------------------------------
   #let's add a user with User provisioning API
   added <- OCS$addUser("john.doe", password = "ocs4john") #add a user
   #let's share the previously uploaded file with John Doe
   OCS$shareWithUser("/Documents", filename, "john.doe")
}

}
\author{
Emmanuel Blondel <emmanuel.blondel1@gmail.com>
}
\keyword{manager}
\keyword{ocs}
\section{Super class}{
\code{\link[ocs4R:ocs4RLogger]{ocs4R::ocs4RLogger}} -> \code{ocsManager}
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{apis}}{list of APIs}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-ocsManager-new}{\code{ocsManager$new()}}
\item \href{#method-ocsManager-connect}{\code{ocsManager$connect()}}
\item \href{#method-ocsManager-getVersion}{\code{ocsManager$getVersion()}}
\item \href{#method-ocsManager-getCapabilities}{\code{ocsManager$getCapabilities()}}
\item \href{#method-ocsManager-getAPIWebdavManager}{\code{ocsManager$getAPIWebdavManager()}}
\item \href{#method-ocsManager-getAPISharingManager}{\code{ocsManager$getAPISharingManager()}}
\item \href{#method-ocsManager-getAPIUserProvisioningManager}{\code{ocsManager$getAPIUserProvisioningManager()}}
\item \href{#method-ocsManager-clone}{\code{ocsManager$clone()}}
}
}
\if{html}{\out{
<details><summary>Inherited methods</summary>
<ul>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="ERROR"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-ERROR'><code>ocs4R::ocs4RLogger$ERROR()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="INFO"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-INFO'><code>ocs4R::ocs4RLogger$INFO()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="WARN"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-WARN'><code>ocs4R::ocs4RLogger$WARN()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="getClass"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-getClass'><code>ocs4R::ocs4RLogger$getClass()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="getClassName"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-getClassName'><code>ocs4R::ocs4RLogger$getClassName()</code></a></span></li>
<li><span class="pkg-link" data-pkg="ocs4R" data-topic="ocs4RLogger" data-id="logger"><a href='../../ocs4R/html/ocs4RLogger.html#method-ocs4RLogger-logger'><code>ocs4R::ocs4RLogger$logger()</code></a></span></li>
</ul>
</details>
}}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsManager-new"></a>}}
\if{latex}{\out{\hypertarget{method-ocsManager-new}{}}}
\subsection{Method \code{new()}}{
This method is used to instantiate an ocsManager. The user/pwd are
mandatory in order to connect to 'ocs'. The logger can be either NULL, "INFO" 
(with minimum logs), or "DEBUG" (for complete curl http calls logs).

The \code{keyring_backend} can be set to use a different backend for storing 
the user password with \pkg{keyring} (Default value is NULL, meaning the password 
is stored as private field).
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsManager$new(url, user, pwd, logger = NULL, keyring_backend = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{url}}{url}

\item{\code{user}}{user}

\item{\code{pwd}}{pwd}

\item{\code{logger}}{logger type}

\item{\code{keyring_backend}}{keyring back-end. Default is \code{NULL}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsManager-connect"></a>}}
\if{latex}{\out{\hypertarget{method-ocsManager-connect}{}}}
\subsection{Method \code{connect()}}{
Method to connect to 'ocs' and set version/capabilities
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsManager$connect()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsManager-getVersion"></a>}}
\if{latex}{\out{\hypertarget{method-ocsManager-getVersion}{}}}
\subsection{Method \code{getVersion()}}{
Get the 'ocs' server version
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsManager$getVersion()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsManager-getCapabilities"></a>}}
\if{latex}{\out{\hypertarget{method-ocsManager-getCapabilities}{}}}
\subsection{Method \code{getCapabilities()}}{
Get the 'ocs' server capabilities
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsManager$getCapabilities()}\if{html}{\out{</div>}}
}

}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsManager-getAPIWebdavManager"></a>}}
\if{latex}{\out{\hypertarget{method-ocsManager-getAPIWebdavManager}{}}}
\subsection{Method \code{getAPIWebdavManager()}}{
Get the Webdav API manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsManager$getAPIWebdavManager()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an instance of \link{ocsApiWebdavManager}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsManager-getAPISharingManager"></a>}}
\if{latex}{\out{\hypertarget{method-ocsManager-getAPISharingManager}{}}}
\subsection{Method \code{getAPISharingManager()}}{
Get the Sharing API manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsManager$getAPISharingManager()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an instance of \link{ocsApiSharingManager}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsManager-getAPIUserProvisioningManager"></a>}}
\if{latex}{\out{\hypertarget{method-ocsManager-getAPIUserProvisioningManager}{}}}
\subsection{Method \code{getAPIUserProvisioningManager()}}{
Get the User Provisioning API manager
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsManager$getAPIUserProvisioningManager()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
an instance of \link{ocsApiUserProvisioningManager}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocsManager-clone"></a>}}
\if{latex}{\out{\hypertarget{method-ocsManager-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocsManager$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}

% Generated by roxygen2: do not edit by hand % Please edit documentation in R/ocsApiWebdavManager.R \docType{class} \name{ocsApiWebdavManager} \alias{ocsApiWebdavManager} \title{ocsApiWebdavManager} \format{ \code{\link[R6]{R6Class}} object. } \value{ Object of \code{\link[R6]{R6Class}} for modelling an ocsManager for Webdav API } \description{ ocsApiWebdavManager ocsApiWebdavManager } \author{ Emmanuel Blondel } \keyword{api} \keyword{manager} \keyword{ocs} \keyword{webdav} \section{Super classes}{ \code{\link[ocs4R:ocs4RLogger]{ocs4R::ocs4RLogger}} -> \code{\link[ocs4R:ocsManager]{ocs4R::ocsManager}} -> \code{ocsApiWebdavManager} } \section{Methods}{ \subsection{Public methods}{ \itemize{ \item \href{#method-ocsApiWebdavManager-new}{\code{ocsApiWebdavManager$new()}} \item \href{#method-ocsApiWebdavManager-getWebdavRoot}{\code{ocsApiWebdavManager$getWebdavRoot()}} \item \href{#method-ocsApiWebdavManager-listFiles}{\code{ocsApiWebdavManager$listFiles()}} \item \href{#method-ocsApiWebdavManager-makeCollection}{\code{ocsApiWebdavManager$makeCollection()}} \item \href{#method-ocsApiWebdavManager-uploadFile}{\code{ocsApiWebdavManager$uploadFile()}} \item \href{#method-ocsApiWebdavManager-deleteFile}{\code{ocsApiWebdavManager$deleteFile()}} \item \href{#method-ocsApiWebdavManager-downloadFile}{\code{ocsApiWebdavManager$downloadFile()}} \item \href{#method-ocsApiWebdavManager-getPublicFile}{\code{ocsApiWebdavManager$getPublicFile()}} \item \href{#method-ocsApiWebdavManager-clone}{\code{ocsApiWebdavManager$clone()}} } } \if{html}{\out{
Inherited methods

- `ocs4R::ocs4RLogger$ERROR()`
- `ocs4R::ocs4RLogger$INFO()`
- `ocs4R::ocs4RLogger$WARN()`
- `ocs4R::ocs4RLogger$getClass()`
- `ocs4R::ocs4RLogger$getClassName()`
- `ocs4R::ocs4RLogger$logger()`
- `ocs4R::ocsManager$connect()`
- `ocs4R::ocsManager$getAPISharingManager()`
- `ocs4R::ocsManager$getAPIUserProvisioningManager()`
- `ocs4R::ocsManager$getAPIWebdavManager()`
- `ocs4R::ocsManager$getCapabilities()`
- `ocs4R::ocsManager$getVersion()`

}} \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ocsApiWebdavManager-new}{}}} \subsection{Method \code{new()}}{ Initialize manager \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ocsApiWebdavManager$new(url, user, pwd, logger = NULL, keyring_backend = NULL)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{url}}{url} \item{\code{user}}{user} \item{\code{pwd}}{pwd} \item{\code{logger}}{logger} \item{\code{keyring_backend}}{backend to use with \pkg{keyring}. Default is \code{NULL}} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ocsApiWebdavManager-getWebdavRoot}{}}} \subsection{Method \code{getWebdavRoot()}}{ Get the 'ocs' WebDAV root URL \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ocsApiWebdavManager$getWebdavRoot()}\if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ocsApiWebdavManager-listFiles}{}}} \subsection{Method \code{listFiles()}}{ WebDAV method to list folders/files given a relative path. The relative path is set to \code{"/"} by default, which corresponds to the root of the 'ocs' repository. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ocsApiWebdavManager$listFiles(relPath = "/")}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{relPath}}{relative path} } \if{html}{\out{
}} } \subsection{Returns}{ the list of files } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ocsApiWebdavManager-makeCollection}{}}} \subsection{Method \code{makeCollection()}}{ WebDAV method to make a collection. By default \code{relPath} is set to \code{"/"} (root). The \code{name} is the name of the new collection to be created. The function is recursive in the sense that a \code{name} can be provided as relative path of a collection tree (eg \code{newfolder1/newfolder2/newfolder3}), the function will create recursively as many collections are handled in the name. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ocsApiWebdavManager$makeCollection(name, relPath = "/")}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{name}}{name} \item{\code{relPath}}{relative path} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ocsApiWebdavManager-uploadFile}{}}} \subsection{Method \code{uploadFile()}}{ WebDAV method to upload a file. By default \code{relPath} is set to \code{"/"} (root). \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ocsApiWebdavManager$uploadFile( filename, relPath = "/", delete_if_existing = FALSE )}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{filename}}{file name} \item{\code{relPath}}{relative path} \item{\code{delete_if_existing}}{delete if existing file? Default is \code{FALSE}} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ocsApiWebdavManager-deleteFile}{}}} \subsection{Method \code{deleteFile()}}{ WebDAV method to delete a file. By default \code{relPath} is set to \code{"/"} (root). \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ocsApiWebdavManager$deleteFile(filename, relPath = "/")}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{filename}}{file name} \item{\code{relPath}}{relative path} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ocsApiWebdavManager-downloadFile}{}}} \subsection{Method \code{downloadFile()}}{ Downloads a file \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ocsApiWebdavManager$downloadFile(relPath, filename, outdir = ".")}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{relPath}}{relative path} \item{\code{filename}}{file name} \item{\code{outdir}}{the out directory where to download the file} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ocsApiWebdavManager-getPublicFile}{}}} \subsection{Method \code{getPublicFile()}}{ Get details of a shared public file given its share token \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ocsApiWebdavManager$getPublicFile(share_token)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{share_token}}{the share token} } \if{html}{\out{
}} } } \if{html}{\out{

}} \if{html}{\out{}} \if{latex}{\out{\hypertarget{method-ocsApiWebdavManager-clone}{}}} \subsection{Method \code{clone()}}{ The objects of this class are cloneable with this method. \subsection{Usage}{ \if{html}{\out{
}}\preformatted{ocsApiWebdavManager$clone(deep = FALSE)}\if{html}{\out{
}} } \subsection{Arguments}{ \if{html}{\out{
}} \describe{ \item{\code{deep}}{Whether to make a deep clone.} } \if{html}{\out{
}} } } }

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ocs4R.R
\docType{package}
\name{ocs4R}
\alias{ocs4R}
\alias{ocs4R-package}
\title{Interface to 'OCS' REST API}
\description{
Provides an Interface to 'OCS' (<https://www.open-collaboration-services.org/>) REST API.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/eblondel/ocs4R}
  \item Report bugs at \url{https://github.com/eblondel/ocs4R/issues}
}

}
\author{
Emmanuel Blondel \email{emmanuel.blondel1@gmail.com}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ocs4R_logger.R
\docType{class}
\name{ocs4RLogger}
\alias{ocs4RLogger}
\title{ocs4RLogger}
\format{
\code{\link[R6]{R6Class}} object.
}
\value{
Object of \code{\link[R6]{R6Class}} for modelling a simple logger
}
\description{
ocs4RLogger

ocs4RLogger
}
\note{
Logger class used internally by ocs4R
}
\keyword{logger}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{verbose.info}}{is info logger active?}

\item{\code{verbose.debug}}{is debug logger active?}

\item{\code{loggerType}}{logger type}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-ocs4RLogger-new}{\code{ocs4RLogger$new()}}
\item \href{#method-ocs4RLogger-logger}{\code{ocs4RLogger$logger()}}
\item \href{#method-ocs4RLogger-INFO}{\code{ocs4RLogger$INFO()}}
\item \href{#method-ocs4RLogger-WARN}{\code{ocs4RLogger$WARN()}}
\item \href{#method-ocs4RLogger-ERROR}{\code{ocs4RLogger$ERROR()}}
\item \href{#method-ocs4RLogger-getClassName}{\code{ocs4RLogger$getClassName()}}
\item \href{#method-ocs4RLogger-getClass}{\code{ocs4RLogger$getClass()}}
\item \href{#method-ocs4RLogger-clone}{\code{ocs4RLogger$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocs4RLogger-new"></a>}}
\if{latex}{\out{\hypertarget{method-ocs4RLogger-new}{}}}
\subsection{Method \code{new()}}{
Initializes the logger
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocs4RLogger$new(logger = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{logger}}{the type of logger. Default is \code{NULL}, accepts \code{INFO} or \code{DEBUG}}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocs4RLogger-logger"></a>}}
\if{latex}{\out{\hypertarget{method-ocs4RLogger-logger}{}}}
\subsection{Method \code{logger()}}{
Logger function
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocs4RLogger$logger(type, text)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{type}}{type of log}

\item{\code{text}}{text}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocs4RLogger-INFO"></a>}}
\if{latex}{\out{\hypertarget{method-ocs4RLogger-INFO}{}}}
\subsection{Method \code{INFO()}}{
Logger to report information. Used internally
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocs4RLogger$INFO(text)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{text}}{text}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocs4RLogger-WARN"></a>}}
\if{latex}{\out{\hypertarget{method-ocs4RLogger-WARN}{}}}
\subsection{Method \code{WARN()}}{
Logger to report warnings. Used internally
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocs4RLogger$WARN(text)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{text}}{text}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocs4RLogger-ERROR"></a>}}
\if{latex}{\out{\hypertarget{method-ocs4RLogger-ERROR}{}}}
\subsection{Method \code{ERROR()}}{
Logger to report errors Used internally
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocs4RLogger$ERROR(text)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{text}}{text}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocs4RLogger-getClassName"></a>}}
\if{latex}{\out{\hypertarget{method-ocs4RLogger-getClassName}{}}}
\subsection{Method \code{getClassName()}}{
Get class name
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocs4RLogger$getClassName()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
the class name
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocs4RLogger-getClass"></a>}}
\if{latex}{\out{\hypertarget{method-ocs4RLogger-getClass}{}}}
\subsection{Method \code{getClass()}}{
Get class
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocs4RLogger$getClass()}\if{html}{\out{</div>}}
}

\subsection{Returns}{
the class
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-ocs4RLogger-clone"></a>}}
\if{latex}{\out{\hypertarget{method-ocs4RLogger-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{ocs4RLogger$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
